<? require "../lib.php"?>
<? aheader("Luca Development Diary - from 0.7 to 0.8"); ?>

<p>
2006/01/26 - Printed reports

<p>
I had some options regarding to printed report support: direct PDF generation, separate
templates for print reports plus a custom CSS, or just a second custom CSS with 'media=print'
directive (so the same report can be made to look completely different in screen and on
paper).

<p>
Generating PDFs give the most good-looking and controllable results, and it would be just
a question of using the ReportLab library. The main problems in this option would be: too
much work to put together, and too difficult for the user to customize even slightly. I still
think there will be PDF reports in the far future, but now the media print CSS seems better.

<p>
Regarding options with CSS, we can create a second report template specific for printed
media (and use the printing CSS style in that), or use the media="print" CSS feature to
have two CSS's for the same page, one for screen and another for print. I didn't know
if this CSS feature would work in real-world browsers (that tend not to implement some
less-often-used CSS features), so I began with the worst option, a separate template.

<p>
I must confess that customizing the CSS for printed media was way easier than I was
expecting. And I could find answers for my doubts very quickly in Google. For example,
a seemingly big problem - printing tables' headers in every page - was solved just by
delimiting the table heading with &lt;thead&gt; and adding a CSS style ("display: table-header-group") 
for thead. Page break on top-level accounts was just as easy (page-break-before style 
for the toplevel-class table row).

<p>
The only thing that needed a bit more of work was to remove titles from the template and
generate them inside the controllers, so the title (which the browser normally prints
at top left) contains extra information about the report e.g. the organization it refers to.
In the end, the custom report templates had so few differences from the screen report
template that I just dropped them. The user simply asks to print the report web page
and CSS magically hides unwanted elements (forms etc.) for the print version.

<p>
The main advantage (and disvantage) of the CSS-based report approach is that its final
appearance on print depends a lot on browser print settings. For example, there is no
CSS way to remove the URL from the top-right of every page, though it is easy to remove
it via print settings. It is a sure thing that most users will report that as a
system bug. In the other hand, relying on browser/CSS relieves us from thinking
about paper size and orientation; it is up to the user to configure those things until
the final format pleases her best. It will be important to emphasize this in a future
user-oriented documentation for Luca.

<p>
Finally, I changed some controllers to the page title tells which data is being
seen (e.g. which chart or which organization). Since the reports have pretty much
the same HTML structure as the data entry controllers, the new print CSS allows
every page in the system to be elegantly printed (e.g. a list of batches or a
chart of accounts).

<p>
2007/02/07 - Database portability 

<p>
Some people from England have contacted me about Luca, and mentioned that they tried to
run it using SQLite but could not do it. It was the "waking call" to test the database
interoperability.

<p>
The biggest issue was to learn how to use SQLite and PostgreSQL since I have never used
them. Using a different database in Luca (or any other Turbogears application) is really
like the documentation says: just change the database URI in $application/config/app.cfg.
I have made a quick test of key features under SQLite and Postgres, and no problem arose
in Luca itself.  In the other hand, some bad problems arose in Turbogears and SQLObject... 

<p> In Postgres,
making a select(..., orderBY=-timestamp) did not work since SQLObject passed the minus
sign unchanged and Postgres did not like that for a date/time column. Changing it to
select(orderBy=timestamp).reversed() solved this issue (it made SQLObject to use the
SORT BY timestamp DESC command). So why not to use DESC for *all* inverse-sorted queries?

<p>
Another issue in Postgres, not a bug, but something you must be aware, is that 
auto-increment IDs are implemented by nextval() SQL functions. If you dump a 
database, say, in MySQL and upload it in Postgres, it will not update the 
ID counters, and adding new rows will likely fail because they will be given
IDs that are already in the database. (I had this problem because I used my own
accounting coming from MySQL in the tests).

<p>
In short, PostgreSQL will work fine with Luca. The problem was more serious with 
SQLite, because of bugs in SQLObject and (perhaps) Turbogears, to the point that
SQLite is (by now) not recommended to be used with Luca.

<p>
One bug showed up in journal entries page, since it made two parallel JSON requests,
and it raised "Database is Locked" errors. I changed the Javasscript code in order
to serialize the two requests, but there still can be problems if there are two
or more simultaneous users. Since the error shows up around identity visit handling,
I guess it is a Turbogears bug, or at least Turbogears is the place to work around
this.

<p>
The really serious SQLite-related bug is inside SQLObject is some caching bug, sometimes
the row objects return old data! Thank God I noted it early: I saw that AJAX-enabled
pages were being reloaded too often and some debugging showed that sometimes old 
data showed up. The bug has been reported in Turbogears Trac with a suggested patch,
you should take a look in it if you really want to use SQLite 
(http://trac.turbogears.org/ticket/1154).

<p>
Last, SQLite does not enforce foreign keys in deletions, and Luca relies on that in
some controllers. This is "our" bug, we should double check for that before trying
to remove rows. So when the abovementioned bugs are fixed, people can safely use
Luca on SQLite.

<p>
2007/02/17 - Improved testing coverage

<p>
After leaving the automatic tests abandoned for quite some time, I felt I needed to 
work again on them. The perspective of other people working on the same codebase 
scared me a bit, and an automated test becomes utterly needed.

<p>
First, the last time I touched the tests, they still did not work with more
complex pages like journals, accounts of the chart and batches. Having put them
to work, I felt the need of coverage statistics. In a quick Google search I found
a very easy-to-use Python coverage tool that I have included in Luca's source
tree (coverage.py).

<p> Follows the first statistics:


<pre>
/home/epx/luca $ ./coverage.py -r luca/*.py
Name                    Stmts   Exec  Cover
-------------------------------------------
luca/__init__               0      0   100%
luca/active_period         90     75    83%
luca/balance_closing      191     26    13%
luca/batches              208    128    61%
luca/charts               100     77    77%
luca/charts_accounts      250    163    65%
luca/controllers           74     47    63%
luca/costcenters          109     85    77%
luca/journals             424    234    55%
luca/json                   1      1   100%
luca/model                250    244    97%
luca/mylib                383    238    62%
luca/orgs                  95     85    89%
luca/parentXchild         126     91    72%
luca/release                7      0     0%
luca/report_balance       138     33    23%
luca/report_cashflow      157     31    19%
luca/report_income        120     29    24%
luca/report_incomeavg     123     29    23%
luca/report_journal       128     31    24%
luca/report_ledger        151     31    20%
luca/reportlib             87      5     5%
luca/simple_table         309    228    73%
luca/stdjournals          111     84    75%
luca/sys_classes           73     66    90%
luca/sys_cmos              67     34    50%
luca/sys_groups            75     68    90%
luca/sys_groupsXperms      41     38    92%
luca/sys_password          60     35    58%
luca/sys_qlogs            154     34    22%
luca/sys_users            138    120    86%
luca/sys_usersXgroups      41     33    80%
luca/sys_usersXorgs        32     31    96%
luca/threelevel_table     106     25    23%
luca/twolevel_table       105     23    21%
-------------------------------------------
TOTAL                    4524   2502    55%
</pre>

<p>
Not *that* bad, but most of the lines covered in untested pages like reports were definitions,
so we needed to work in order to improve coverage to 85% at least.

<p>
A lot of work has been put in controller testing, to the point that the current testing routine
needs to be refactored: it became too big and full of exceptional handling for specific controllers.  
The "exceptional handling" must be moved to each controller's source file, but it is something
to be done in the near future, not now.

<p>
Testing of "lite" pages brought some funny problems. Lite pages use a lot of redirect()
exceptions (that redirect the user to another page etc.), and that exception was not
well handled by Turbogears while in test mode. A patch found Turbogears' Trac solved this
particular problem, but other issues showed that it would be better if the method behaved
a bit different under testing, that is, returning the error message instead of redirecting
(because redirection cannot tell if it was due to error or success). 

<p>So, lite_edit and lite_commit methods now have an optional _testing_ parameter that is
passed = True when called from testing program. Fortunately, former refactoring efforts
assured that "lite" and AJAX methods behave almost exactly the same (save for redirections)
and share almost 100% of the common code, so the _testing_ parameter does not imply in 
reduced coverage.

<p>
Another major issue was testing the balance closing. It did not suffice to run the routine
with a few journal entries, it needed to be a big journal with known results to be compared
against.

<p>
So I got the database from the site's live version, calculated the balance closing batch,
changed the batch date to the future, and dumped the database. Now the test closes the 
balance and compares the closing batch with the "old" closing batch that is included in
the data dump, and reports failure if there is any difference.

<p>
Since almost all reports (balance, income statement, cash flow) share the calculation routines
with balance closing, using a big sample database ensures that all these function points have
now a fine regression test.

<p> 
As the test-driven advocates predict, our tests found very subtle bugs, e.g. a security
hole in reports (everyone would be able to print any report before that). 

<p>
The final coverage report (for version 0.8 final):


<pre>
Name                    Stmts   Exec  Cover
-------------------------------------------
luca/__init__               0      0   100%
luca/active_period         92     85    92%
luca/balance_closing      182    164    90%
luca/batches              206    181    87%
luca/charts               100     89    89%
luca/charts_accounts      245    240    97%
luca/controllers           55     49    89%
luca/costcenters          109    108    99%
luca/journals             414    335    80%
luca/json                   1      1   100%
luca/model                245    244    99%
luca/mylib                376    311    82%
luca/orgs                  93     91    97%
luca/parentXchild         114     90    78%
luca/release                7      0     0%
luca/report_balance       133    128    96%
luca/report_cashflow      152    140    92%
luca/report_income        115    109    94%
luca/report_incomeavg     118    112    94%
luca/report_journal       124    118    95%
luca/report_ledger        147    137    93%
luca/reportlib             87     80    91%
luca/simple_table         292    274    93%
luca/stdjournals          111    110    99%
luca/sys_classes           73     73   100%
luca/sys_cmos              67     34    50%
luca/sys_groups            74     74   100%
luca/sys_groupsXperms      41     40    97%
luca/sys_password          65     62    95%
luca/sys_qlogs            138    113    81%
luca/sys_users            134    130    97%
luca/sys_usersXgroups      41     40    97%
luca/sys_usersXorgs        32     31    96%
luca/threelevel_table      97     71    73%
luca/twolevel_table        98     79    80%
-------------------------------------------
TOTAL                    4378   3943    90%
</pre>

<p>
Even though we have marked a lot of code as unreachable (thus they are not taken into account in the 
statistics), there are still a lot of conditions that are not tested, and will be added steadily as
I have the patience to do that :) In the far future, code marked as unreachable, for example SQL
error exceptions, should be truly covered too, by simulation of a SQL failure (not easy and not
urgent, though).


<? footer("/", "en"); ?>


