import cherrypy

import turbogears
from turbogears import controllers, expose, validate, redirect
from turbogears import identity, validators

from sqlobject import sqlbuilder

from luca.model import cmos, conn, notfound, denied
from luca import model

from luca.simple_table import simple_table

import mylib

class active_period(simple_table):

# SUBCLASS-SPECIFIC FUNCTIONS AND PARAMETERS ##########################################################
	
    validators = {"id": validators.Int(min=1, if_invalid=0), 
        	  "active_d1": validators.UnicodeString(if_invalid=u""),
        	  "active_d2": validators.UnicodeString(if_invalid=u""),
	          }
				 
    def setup(self):
	self.emptyrow_read  = {"id": 0, "active_d1": "", "active_d2": ""}
	self.emptyrow_write = self.emptyrow_read
	self.empty_intermediate = self.emptyrow_read.copy()
	self.empty_intermediate["name"] = ""

	# data for automated tests
	self.sample_data = [{"active_d1": "2006/03/31", "active_d2": "2006/01/01"}, \
				{"active_d1": "2004/02/29", "active_d2": "2006/03/31"}]
	self.invalid_data = [("active_d1", "sadfjl"), ("active_d2", "sdakl"), \
			     ("active_d1", "2006/02/29"), ("active_d2", "2006/02/29")]
	self.unique_cols = []
	self.foreign_keys = []
	self.test_order = 6
	self.test_flags = ('noorg_index_allowed','cant_insert')
	self.test_bump = [ ]

    	self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
    	self.msg_notfound = "Organization not found, probably it has just been deleted"
	self.show_msg_ajax = True
    	self.msg_added = "Organization %d has been added"
    	self.msg_updated = "Organization %d has been updated"
    	self.msg_removed = "Organization %d has been removed sucessfully"
	self.msg_validation = "Order number and name must not be empty"
    	self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
    	self.msg_integrity = "Integrity error: "
    	self.msg_cannotremove = "Organization can not be removed"
    	self.msg_cannotchangechart = "Organization already has batches/journal entries - cannot change chart!"
    	self.msg_unique = "check if the order number is unique"
    	self.msg_legend_update = "Organizations: editing organization %s"
    	self.msg_legend_add = "Organizations: new organization"
    	self.msg_chartnotfound = "Chart not found in database!"

    	self.table = model.orgs
	self.cmos = cmos

    def test__verify_excuse(self, colname, orig, fetched, orig_row, fetched_row): #pragma: no cover
	return 1

    def row_name(self, d_row):
	# we cannot return the name since the name does not come from the form when editing
	name = u"%s (id %d)" % (self.table.get(d_row["id"]).name, d_row["id"])
	return name

    def row_to_intermediate_ext(self, row, intermediate):
	intermediate["active_d1"] = mylib.dtoc(row["active_d1"])
	intermediate["active_d2"] = mylib.dtoc(row["active_d2"])
	intermediate["name"] = ""
	if intermediate["id"] > 0:
		intermediate["name"] = self.table.get(intermediate["id"]).name

    def intermediate_to_row_ext(self, intermediate, row):
	row["active_d1"] = mylib.ctod(intermediate["active_d1"])
	row["active_d2"] = mylib.ctod(intermediate["active_d2"])

    def intermediate_validate(self, d):
	if d["id"] <= 0: #pragma: no cover
		# second-level paranoid protection
		return "This controller can not add new rows"
	if mylib.ctod(d["active_d1"]) is None:
		return "Invalid initial date"
	if mylib.ctod(d["active_d2"]) is None:
		return "Invalid final date"

	if mylib.ctod(d["active_d1"]) > mylib.ctod(d["active_d2"]):
		tmp = d["active_d1"]
		d["active_d1"] = d["active_d2"]
		d["active_d2"] = tmp

	for batch in model.batches.select(sqlbuilder.AND(sqlbuilder.SQLConstant("org_id = %d" % d["id"]), model.batches.q.closed <= 0)):
		if batch.date1 < mylib.ctod(d["active_d1"]) or batch.date2 > mylib.ctod(d["active_d2"]):
			return "Batch #%d is open and falls outside the new active period - close the batch first" % batch.id
	return ""
		   
    def data_legend(self, d):
	    return unicode(d["id"])+u" - "+d["name"]

    def perm_org(self, interm):
	return interm["id"]

    def base_update(self, row):
	if row["id"] <= 0: #pragma: no cover
		# Last resort paranoid protection
		raise Exception("This controller can not add new rows")
	return super(active_period, self).base_update(row)

# EXPOSED METHODS ########################################################

    @expose(template="luca.templates.active_period")
    def index(self, **dummy):
        return self.base_index(orderBy=self.table.q.name)

    @expose(template="luca.templates.active_period_lite")
    def lite(self, **dummy):
        return self.base_index(orderBy=self.table.q.name)

    @expose("json")
    @validate(validators=validators)
    def fetch(self, id, **garbage):
	    if id <= 0:
		return {"err": "id must be above zero"}
	    return self.base_fetch(id)

    @expose(template="luca.templates.active_period_lite_form")
    @validate(validators=validators)
    def lite_edit(self, id, **garbage):
	    if id <= 0:
		self.flash_at_index(self.msg_validation_crash, err=1)
		if model._testing_:
			return {"err": self.msg_validation_crash}
		raise redirect("lite") #pragma: no cover
	    return self.base_lite_edit(id)

    @expose("json")
    @validate(validators=validators)
    def update(self, **form_data):
	    if form_data["id"] <= 0:
		return {"err": "id must be above zero"}
	    return self.base_update(form_data)

    @expose(template="luca.templates.active_period_lite_form")
    @validate(validators=validators)
    def lite_commit(self, **form_data):
	    if form_data["id"] <= 0:
		self.flash_at_index(self.msg_validation_crash, err=1)
		if model._testing_:
			return {"err": "ble"}
		raise redirect("lite") #pragma: no cover
	    return self.base_lite_commit(form_data)

