import turbogears
from turbogears import controllers, redirect, flash
from turbogears import identity, validators, expose, validate

from luca.model import sqlerror, notfound, denied, log, hub, ModelError, gconn
from luca import model
import os
import sys

from sqlobject import sqlbuilder

import mylib
import reportlib
import datetime
import time

class balance_closing(controllers.Controller):

# MODEL- AND LANGUAGE-SPECIFIC CODE & MESSAGES ######################################

    validators = { \
	          "date1": validators.UnicodeString(if_invalid=u""),
	          "date2": validators.UnicodeString(if_invalid=u""),
		  "org": validators.Int(if_invalid=0),  \
		  "submitted": validators.Int(if_invalid=0) }
				 
    def setup(self):
 	   self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
           self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
           self.msg_validation = self.msg_validation_crash


           self.valid_requests = [ dict(org=1, date1="2006/01/31", date2="2006/01/01", submitted=1), \
          			   dict(org=1, date1="2006/01/31", date2="2006/01/01", submitted=2), \
          			   dict(org=1, date1="2006/01/31", date2="2006/01/01", submitted=3) ]

           self.invalid_requests = [ dict(org=1, date1="2006/10/01", date2="2006/10/30", submitted=1), \
				dict(org=1, date1="2006", date2="2006/01/31", submitted=1), \
				dict(org=1, date1="2006/01/01", date2="x", submitted=1), \
				dict(org=1, date1="2006/04/01", date2="2006/04/30", submitted=1), \
				dict(org=1, date1="2005/11/01", date2="2005/11/30", submitted=1), \
           			dict(org=-1, date1="2006/01/31", date2="2006/01/01", submitted=2), \
           			dict(org=999, date1="2006/01/31", date2="2006/01/01", submitted=2), \
           			dict(org=0, date1="2006/01/31", date2="2006/01/01", submitted=2), \
				dict(org=1, date1="", date2="", submitted=1), \
				dict(org=1, date1="2006", date2="2006/01/31", submitted=2), \
				dict(org=1, date1="2006/01/01", date2="x", submitted=2), \
				dict(org=1, date1="", date2="", submitted=2) ]

	   self.invalid_requests2 = [ dict(org=1, date1="2006/01/01", date2="2006/01/31", submitted=2), \
				dict(org=1, date1="2006/01/01", date2="2006/01/31", submitted=2)  ]
	   self.test_flags = ()
	   self.vips = []

 
    def sqlerr(self, exception): #pragma: no cover
	    self.log.error("%s SQL error %s", self.__class__.__name__, str(exception.args))
	    return self.msg_unexpected

    def test__index(self, t, data): #pragma: no cover #pragma: no cover
	t.cargo = t.tu.call(self.index, **data)
	if not t.cargo["err"] or t.cargo["err"] == "No log entries found":
		t.cargo = None
		return True
	t.cargo = None
	return False
	
    def test__run(self, t): #pragma: no cover
	t.C = self

	t.logmask = 0

	t.forgive_permission_hole = 1
	t.forgive_org_permission_hole = 1
	t.logmask = t.DONE + t.FAIL + t.DENIED

	# FIXME: dependent extras/luca.testdata.dump.alt contents!
	c = dict(batch_zero=1, batch_open=1, accounts_checkdigit=1)
	model.cmos.get(1).set(**c)
	model.orgs.get(1).set(chart=1, active_d1="2005-01-01", active_d2="2006-05-31")
	model.charts.get(1).set(mask='9.9.9.99.9999')

	# os.system("cat extra/luca.testdata.dump | mysql luca -u root --password=''")

	data = file("extra/luca.testdata.dump.alt").readlines()
	data = [ eval(line) for line in data if type(eval(line)) is tuple  ]

	skel = None
	tclass = None
	nclass = None
	for l in data:
		if l[0] == 'table':
			skel = l
			nclass = l[1]
			tclass = getattr(model, nclass)
			[ row.delete(row.id) for row in tclass.select() ]
		elif l[0] == 'row':
			if l[1] != nclass: #pragma: no cover
				raise Exception("tclass != nclass")
			row = {}
			for i in range(2, len(l)):
				row[skel[i]] = l[i]
			print nclass, row
			tclass(**row)

	res = t.permission_explode("empty page", self.test__index, t.DONE, {})
	if not res:
		t.fail("Failed")

	t.forgive_permission_hole = 0
	t.forgive_org_permission_hole = 0
	t.logmask = 0

	for req in t.C.invalid_requests:
		res = t.permission_explode("invalid closing", self.test__index, t.DONE, req)
		if res and not t.forgive_permission_hole:
			t.fail("Should have failed")

	for req in t.C.valid_requests:
		t.logmask = 0
		if req['submitted'] != 2:
			t.logmask = t.DONE
		if req['submitted'] == 3:
			t.logmask = t.DENIED + t.DONE
			t.forgive_permission_hole = 1
			t.forgive_org_permission_hole = 1
		res = t.permission_explode("closing", self.test__index, t.DONE, req)
		if not res:
			t.fail("Failed")

	t.logmask = 0
	t.forgive_permission_hole = 0
	t.forgive_org_permission_hole = 0
	for req in t.C.invalid_requests2:
		res = t.permission_explode("invalid closing post", self.test__index, t.DONE, req)
		if res and not t.forgive_permission_hole:
			t.fail("Should have failed")

	t.forgive_permission_hole = 0

	j = model.batches.select(sqlbuilder.AND(sqlbuilder.SQLConstant("org_id = 1"), \
						sqlbuilder.AND(model.batches.q.date1 == '2006-01-01', \
							      model.batches.q.date2 == '2006-01-31'), \
					       sqlbuilder.SQLConstant("description LIKE '%[clo]%'")))

	generated_batch_id = 0
	print "Discovering the newest batch that closes balance"
	for b in j:
		print "... batch = %d" % b.id
		generated_batch_id = b.id

	if not generated_batch_id:
		t.fail("No generated journals")

	generated_batch = model.batches.get(generated_batch_id).journals
	model_batch = model.batches.get(137).journals

	if not model_batch:
		t.fail("No model journals")
	
	while generated_batch:
		g = generated_batch[0]
		if not model_batch:
			t.fail("Ran out of models (%s)" % str(g))
		for x in xrange(0, len(model_batch)):
			m = model_batch[x]
			if m.account.id == g.account.id and m.value == g.value:
				del model_batch[x]
				del generated_batch[0]
				break
		else:
			t.fail("Generated journal (%s) does not match any model" % str(g))
	if generated_batch:
		t.fail("Generated journals (%s) unmatched" % str(generated_batch))
	if model_batch:
		t.fail("Model journals (%s) unmatched" % str(model_batch))
	
    def __init__(self, log):
        super(balance_closing, self).__init__()
	self.setup()
    	self.log = log




    def check(self, org, date1, date2):
	err = ""
	msg = ""
	date1 = mylib.ctod(date1)
	date2 = mylib.ctod(date2)
	account = None

	try:
		if date1 is None:
			raise ModelError("Initial date is invalid")
		elif date2 is None:
			raise ModelError("Final date is invalid")
		elif org <= 0:
			raise ModelError("Please choose an organization")
	
		if date1 > date2:
			date1, date2 = date2, date1
	
		try:
			org = model.orgs.get(org)
			msg = org.remarks
		except notfound, e:
			raise ModelError("Organization does not exist (just deleted?)")
	
		if date1 < org.active_d1 or date2 > org.active_d2:
			raise ModelError("Closing period is outside organization's active period")
	
		batches = {}
		days = {}
		mov_income = 0

		j = model.batches.select(sqlbuilder.AND(sqlbuilder.SQLConstant("org_id = %d" % org.id), \
				 sqlbuilder.OR(sqlbuilder.AND(model.batches.q.date1 >= date1, model.batches.q.date1 <= date2), \
						sqlbuilder.AND(model.batches.q.date2 <= date2, model.batches.q.date2 >= date1)),
					                sqlbuilder.SQLConstant("description LIKE '%[clo]%'")))

		for row in j:
			# detected closing batch in our closing period
			raise ModelError(u"There is a previous balance closing that falls within the closing period (Batch #%d)." % row.id)
	
		j = model.journal.select(sqlbuilder.AND( \
					  model.journal.q.day >= date1, model.journal.q.day <= date2, \
					  model.journal.q.cancelled <= 0, \
		 		   	  sqlbuilder.SQLConstant("org_id = %d" % org.id)), \
		 			  orderBy=(model.journal.q.day) )
	
		for row in j:
			day = mylib.dtoc(row.day)
			if int(row.value) == 0:
				raise ModelError(u"Journal batch #%d ID #%d is zero-valued (unexpected error)" % (row.batch.id, row.id))
			if not row.batch.closed:
				raise ModelError(u"Batch #%d is not closed. Close all batches before closing balance" % row.batch.id)
			if row.batch.closed and row.batch.adjust and u"[clo]" in row.batch.description:
				raise ModelError(u"There is a previous balance closing that falls within the closing period (Batch #%d, Day %s)." % (row.batch.id, day))
			if row.account.xclass.is_income:
				mov_income += abs(int(row.value))
			try:
				days[day] += int(row.value)
			except KeyError:
				days[day] = int(row.value)
			try:
				batches[row.batch.id][day] += int(row.value)
			except KeyError:
				try:
					batches[row.batch.id][day] = int(row.value)
				except KeyError:
					batches[row.batch.id] = {}
					batches[row.batch.id][day] = int(row.value)
	
		if not days:
			raise ModelError(u"Closing period has no entries, and it is not allowed to close an empty balance")
	
		if not mov_income:
			raise ModelError(u"Income movement of the given period is zero. It is not allowed to close a balance with no income and no expenses.")
	
		for id in batches.keys():
			batch = batches[id]
			for day in batch.keys():
				value = batch[day]
				if value != 0:
					raise ModelError(u"Batch %d Day %s has uneven debit/credit" % (batch_id, day))
	
		for day in days.keys():
			value = days[day] 
			if value != 0:
				raise ModelError(u"Day %s has uneven debit/credit (unexpected!)" % day)
	
		q = model.accounts.select(sqlbuilder.AND( \
					  sqlbuilder.SQLConstant("description LIKE '%[clo]%'"), \
		 		   	  sqlbuilder.SQLConstant("chart_id = %d" % org.chart.id)))
	
		account = None
		for a in q:
			if not account:
				account = a
			else:
				raise ModelError(u"More than one profit/loss account was found in the chart (description with '[clo]' tag)!")
	
		if account is None:
			raise ModelError(u"Accumulated profit/loss account was not found in the chart (description with '[clo]' tag)")
	
	except ModelError, e:
		err = e.args[0]
	except sqlerror, e: #pragma: no cover
		err = self.sqlerr(e)+" (SQL Error) "
	
	return (err, msg, org, account, date1, date2)
	
	
    def query(self, org, date1, date2):
	err = ""

	err, dummy, rorg, closing_account, ddate1, ddate2 = self.check(org, date1, date2)
	if err:
		return err

	mylib.setdefault(self, "org", org, 0)

	try:
		model.hub.commit()
		raw_report = reportlib.balance_report(rorg.id, ddate1, ddate2, 0, 0, 2, 0)
	except sqlerror, e: #pragma: no cover
		err = self.sqlerr(e) + "(while calculating incoming statement)"
		return err

	if raw_report is None:
		err = "Income statement not calculated - chart of accounts has problems"
		return err

	checksum = 0
	rows = []

	for row in raw_report:
		row["liq"] = row["cr"] - row["db"]
		if row["liq"] != 0 and not row["aggregate"] and row["is_income"]:
			rows.append(row)
			checksum += abs(row["liq"])

	if not checksum:
		err = u"Unexpected error: no journal entries in the period!"

	try:
		needs_rollback = False
		hub.commit()
		needs_rollback = True

		description = u"Balance closing batch [clo]"

		batch = model.batches(org=rorg, date1=ddate1, date2=ddate2, total=checksum, \
				      closed=1, adjust=1, description=description)

		for row in rows:
			r = {"org": rorg.id, "batch": batch.id, "account": row["account_id"], "value": -row["liq"], \
			     "day": ddate2, "description": description, "stdjournal": 1, \
			     "costcenter": 1}
			s = r.copy()
			s["account"] = closing_account.id
			s["value"] = -s["value"]
			r["hash"] = model.journal_hash(r)
			s["hash"] = model.journal_hash(s)
			r["transa"] = None
			s["transa"] = None
			model.journal(**r)
			model.journal(**s)

		hub.commit()
		needs_rollback = False
		gconn.cache.clear()

	except sqlerror, e: #pragma: no cover
		err = self.sqlerr(e) + " (closing balance)"
		if needs_rollback: 
			hub.rollback()
	
	return err


    def select_tables(self):
	orgs = [{"id": 0, "name": "---"}]
	err = ""

	try:
		for row in model.orgs.select(orderBy=model.orgs.q.name):
			orgs.append({"id": row.id, "name": row.name})
	except sqlerror, e: #pragma: no cover
		err = self.sqlerr(e) + "(reading organizations)"

	return (orgs, err)

    def perm_org(self, d):
	return d["org"]

    @expose(template="luca.templates.balance_closing")
    @identity.require(identity.not_anonymous())
    @validate(validators=validators)
    def index(self, org=0, date1="", date2="", submitted=0, **dummy):
	err = ""
	msg = ""
	try:
		if submitted == 0:
			submitted = 1
		elif submitted == 1:
			grant = model.permission(self, "write.add", org,  model.permission.MASK_DONE)
			err, msg, dummy1, dummy2, dummy3, dummy4 = self.check(org, date1, date2)
			if err:
				grant.fail()
			else:
				submitted = 2
				grant.done()
		elif submitted == 2:
			grant = model.permission(self, "write.add", org, 0)
			err = self.query(org, date1, date2)
			if err:
				grant.fail(err)
			else:
				msg = u"*** Balance closing has been completed succesfully ***"
				date1 = date2 = ""
				submitted = 3
				grant.done()
		elif submitted >= 3: #pragma: no cover
			# already closed, dead-end
			pass
	except denied, e:
		err = e.args[0]

	if err:
		submitted = 1
	
	org = mylib.getdefault(self, "org", org, 0)

	orgs, err2 = self.select_tables()

	if err or err2:
		flash(err or err2)

	return dict(org=org, date1=date1, date2=date2, orgs=orgs, tg_flash_class="errmsg", msg=msg, submitted=submitted, err=err or err2)

    @expose()			#pragma: no cover
    def lite(self, **dummy): 	#pragma: no cover
	redirect("./")		#pragma: no cover

