import cherrypy

import turbogears
from turbogears import controllers, expose, validate, redirect
from turbogears import identity, validators
from sqlobject import sqlbuilder

from luca.model import conn, notfound, sqlerror, denied
from luca import model, mylib

from luca.twolevel_table import twolevel_table

class batches(twolevel_table):

# SUBCLASS-SPECIFIC FUNCTIONS AND PARAMETERS ##########################################################
	
    validatorz = {"id": validators.Int(if_invalid=0), 
		  "org": validators.Int(if_invalid=0),   # mother
        	  "date1": validators.UnicodeString(if_invalid=u""),
        	  "date2": validators.UnicodeString(if_invalid=u""),
        	  "description": validators.UnicodeString(if_invalid=u""),
        	  "total": validators.Int(if_invalid=0),
		  "adjust": validators.Int(if_invalid=0),
		  "closed": validators.Int(if_invalid=0)}

    validatorz_mother = {"mother": validators.Int(if_invalid=0), "id": validators.Int(if_invalid=0)}
				 
    def setup(self):
	self.emptyrow_read = {"id": 0, "org": 0, "date1": "", "date2": "",\
			 "description": "",  "total": 0, "adjust": 0, "closed": 0}
	self.emptyrow_write = self.emptyrow_read
	self.empty_intermediate = self.emptyrow_read

	# data for automated tests
	self.sample_data = [ {"org": 1, "date1": "2006/01/01", "date2": "2006/01/31", "adjust": 0, "closed": 0, "total": 100000},  \
			     {"org": 1, "date1": "2006/03/03", "date2": "2006/03/31", "adjust": 1, "closed": 0, "total": 0 }, \
			     {"org": 1, "date1": "2006/03/03", "date2": "2006/03/31", "adjust": 0, "closed": 0, "total": 0 }, \
			     {"org": 1, "date1": "2006/03/03", "date2": "2006/03/31", "adjust": 1, "closed": 0, "total": 0 }, \
			     {"org": 1, "date1": "2006/03/03", "date2": "2006/03/31", "adjust": 0, "closed": 0, "total": -1 } ]
	self.invalid_data = [ ("description", ""), ("date1", "1.1A"), ("date1", "1."),  \
			      ("date1", "2005/02/29"), ("date1", "2005/01/01"), ("date2", "2005/01/31"),  \
			      ("date2", "bla"), ("date2", "2005/02/30"), ("date2", "2007/12/31"), ("org", 999) ]

	self.invalid_compound_new = [ dict(closed=1) ]

	self.unique_cols = []
	self.foreign_keys = [("orgs", 1)]
	self.test_order = 9
	self.test_bump = [ ('orgs', 'version', 'org') ]

    	self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
    	self.msg_notfound = "Batch not found, probably it has just been deleted"
	self.msg_motherinvalid = "Organization ID invalid (perhaps organization has just been deleted?)"
	self.show_msg_ajax = True
    	self.msg_added = "Batch %d has been added"
    	self.msg_updated = "Batch %d has been updated"
    	self.msg_removed = "Batch %d has been removed sucessfully"
	self.msg_validation = "Invalid data, please fix"
    	self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
    	self.msg_integrity = "Integrity error: "
    	self.msg_cannotremove = "Batch can not be removed, there are journal entries in it"
    	# self.msg_unique = 
    	self.msg_legend_update = "Batches: editing %s"
    	self.msg_legend_add = "Batches: new account"

	self.title = "Journal batches"
	self.title_mask = "Journal batches of %d - %s"

    	self.table = model.batches
	self.mother_table = model.orgs
	self.mother_field_name = "org"

	self.formatters = {"_extra_": self.format_data}

    def test__verify_excuse(self, col, orig, fetched, orig_row, fetched_row): #pragma: no cover
	return col == "total" and fetched == 0 and orig <= 0

    def format_data(self, d):
	d['closed'] = d['closed'] and "Closed" or "Open"

    def row_name(self, d_row):
	org_code = d_row["org"]
	if isinstance(org_code, model.orgs):
		org_code = org_code.id
	return u"%d::%d" % (org_code, d_row["id"])

    def perm_org(self, interm):
	return interm["org"]

    def make_title(self, mother):
	t = self.title
	if mother > 0:
		try:
			t = self.title_mask % (mother, self.mother_table.get(mother).name)
		except sqlerror:  #pragma: no cover
			pass
	return t

    def form_to_intermediate_ext(self, form_data, d):
   	d["description"] = d["description"][:60].strip()
   	d["date1"] = d["date1"][:12].strip()
   	d["date2"] = d["date2"][:12].strip()
	# Cast boolean values into 1/0 field
	for item in ["adjust", "closed"]:
		if d[item]:
			d[item] = 1
		else:
			d[item] = 0
	if d["total"] < 0:
		d["total"] = 0
    
    def intermediate_validate(self, d):
	if not d["description"]:
		return "Description must not be empty"

	if mylib.ctod(d["date1"]) is None:
		return "Invalid initial date"
	if mylib.ctod(d["date2"]) is None:
		return "Invalid final date"

	if mylib.ctod(d["date1"]) > mylib.ctod(d["date2"]):
		tmp = d["date1"]
		d["date1"] = d["date2"]
		d["date2"] = tmp

	if not isinstance(d["org"], model.orgs):
		try:
			org_row = model.orgs.get(d["org"])
		except notfound, e:
			return "Parent org not found (maybe just deleted?)"
	else:
		org_row = d["org"]

	if mylib.ctod(d["date1"]) < org_row.active_d1 or mylib.ctod(d["date2"]) > org_row.active_d2:
		return "Batch would lie outside organization's active period (%s to %s)" % \
			(mylib.dtoc(org_row.active_d1), mylib.dtoc(org_row.active_d2))

	if d["id"] > 0:
		current_batch = model.batches.get(d["id"])
		if current_batch.org.id != org_row.id:
			return "Tried to change batch's organization (NOT ALLOWED!)"
		for journal in current_batch.journals:
			if journal.day < mylib.ctod(d["date1"]) or journal.day > mylib.ctod(d["date2"]):
				return "New date limits conflict with existing journals"
		if d["closed"]:
			err = self.is_batch_even(current_batch, d["total"])
			if err:
				return err
		if current_batch.closed and d["closed"]:
			return "You can not edit a closed batch, reopen it first"
	else:
		if d["closed"]:
			return "A new batch must be open"

	return ""

    def row_to_intermediate_ext(self, row, intermediate):
	    '''Data dictionary manipulation just before editing an existing row'''
	    # Reduces sub-rows to their IDs
	    if isinstance(intermediate["org"], model.orgs):
		intermediate["org"] = intermediate["org"].id
	    intermediate["date1"] = mylib.dtoc(row["date1"])
	    intermediate["date2"] = mylib.dtoc(row["date2"])
	    intermediate["total"] = int(intermediate["total"])

    def intermediate_to_row_ext(self, intermediate, row):
	    row["date1"] = mylib.ctod(intermediate["date1"])
	    row["date2"] = mylib.ctod(intermediate["date2"])

    def data_legend(self, d):
	    return "%d :: %d" %(d["org"], d["id"])

    def get_index_rows(self, mother):
	rows = []
	err = ""
	softerr = ""
	if mother > 0:
		try:
			try:
				org = model.orgs.get(mother)
				accounts_checkdigit = model.cmos.get(1).accounts_checkdigit
			except notfound:
				err = "Organization not found!"
			if not err:
				rows = self.table.select(sqlbuilder.AND( \
							sqlbuilder.SQLConstant("org_id = %d" % org.id), \
							self.table.q.date1 >= org.active_d1, \
							self.table.q.date2 <= org.active_d2), \
							orderBy=self.table.q.date1).reversed()
				# if rows.count() <= 0:
				# 	err = "No batches in the active period for this organization."
		except sqlerror, e:  #pragma: no cover
			err = self.sqlerr(e)
	return (rows, err, softerr, {})

    def auxiliary_form_data(self, form_data):
	err = ""
	form_data["closedlist"] = [{"id": 0, "description": "Open"}, {"id": 1, "description": "Closed"}]
	form_data["adjustlist"] = [{"id": 0, "description": "Normal batch"}, \
			     {"id": 1, "description": "Equity adjustment batch"}]
	form_data["mothers"] = [{"id": 0, "description": "---"}]
	for row in model.orgs.select(orderBy=model.orgs.q.name):
		form_data["mothers"].append({"id": row.id, "description": "%d - %s" % (row.id, row.name)})
	return err

    def trigger_update_post(self, row, id, new, form_data):
	    self.table.get(id).bump_version()
	    mother_row = row["org"]
	    if not isinstance(mother_row, self.mother_table):
		mother_row = self.mother_table.get(row["org"])
	    mother_row.bump_version()

    def trigger_remove_pre(self, row):
	b = self.table.get(row["id"])
	for j in b.journals:
		return "Only empty batches can be removed"
	return ""

    def trigger_remove_post(self, row):
	    mother_row = row["org"]
	    if not isinstance(mother_row, self.mother_table):
		mother_row = self.mother_table.get(row["org"])
	    mother_row.bump_version()

    def table_version(self, row):
	version = -1
	if "org" in row and row["org"] > 0:
	    version = self.mother_table.get(row["org"]).version
	elif "mother" in row and row["mother"] > 0:
	    version = self.mother_table.get(row["mother"]).version
	return version

# MY OWN METHODS ##################################################

    def is_batch_even(self, batch, total):
	total_db = total_cr = 0
	# TODO: that could be optimized by SUM()
	days = {}
	for journal in batch.journals:
		if journal.cancelled > 0:
			continue
		if int(journal.value) > 0:
			total_cr += int(journal.value)
		else:
			total_db -= int(journal.value)
		try:
			days[journal.day] += int(journal.value)
		except KeyError:
			days[journal.day] = int(journal.value)

	# if total_cr + total_db <= 0:
	# 	return "Batch has no journals, can not be closed"

	if total_cr != total_db:
		return "Batch is not even, difference is %s" % mylib.monetary_dbcr(total_cr - total_db)

	if total and total_cr != total:
		return "Journals do not match batch checksum, difference is %s (%s)" % \
	 		(mylib.monetary(abs(total_cr - total)), total_cr > total and "Entries overflow batch" or "Entries less than batch")

	for day in days.keys():
		if days[day] != 0:
			return "Journals of day %s are not balanced" % mylib.dtoc(day)

	return ""

# EXPOSED METHODS ########################################################

    @expose(template="luca.templates.batches")
    @validate(validators=validatorz_mother)
    def index(self, mother=0, **dummy):
        return self.base_index(False, mother)

    @expose(template="luca.templates.batches_lite")
    @validate(validators=validatorz_mother)
    def lite(self, mother=0, **dummy):
        d = self.base_index(True, mother)
	return d

    @expose("json")
    @validate(validators=validatorz_mother)
    def remove(self, id, **garbage):
	    # errors like not closed, has journals etc. are caught by integrity checks
	    result = self.base_remove(id)
	    return result

    @expose()
    @validate(validators=validatorz_mother)
    def lite_remove(self, id, **garbage):
	    return self.base_lite_remove(id)

    @expose("json")
    @validate(validators=validatorz_mother)
    def fetch(self, mother, id, **garbage):
	    return self.base_fetch(mother, id)

    @expose(template="luca.templates.batches_lite_form")
    @validate(validators=validatorz_mother)
    def lite_edit(self, mother, id, **form_data):
	   d = self.base_lite_edit(mother, id)
	   return d

    @expose("json")
    @validate(validators=validatorz)
    def update(self, **form_data):
	    result = self.base_update(form_data)
	    return result

    @expose(template="luca.templates.batches_lite_form")
    @validate(validators=validatorz)
    def lite_commit(self, **form_data):
	    d = self.base_lite_commit(form_data)
            # if method call returns, something was wrong (and error msg is inside d["err"])
	    return d

    def toogle_closed(self, mother, id):
	    form_data = self.base_fetch(mother, id)
	    if form_data["err"]:
		    return form_data
	    if form_data["closed"]:
	    	form_data["closed"] = 0
	    else:
	    	form_data["closed"] = 1
	    form_data["err"] = self.base_update(form_data)["err"]
	    return form_data

    @expose("json")
    @validate(validators=validatorz_mother)
    def closed(self, mother, id, **garbage):
	return self.toogle_closed(mother, id)

    #    @expose()
    #    @validate(validators=validatorz_mother)
    #    def lite_closed(self, mother, id, **garbage):
    #	    form_data = self.toogle_closed(mother, id)
    #	    if form_data["err"]:
    #		    self.flash_at_index("Could not update status: "+form_data["err"], err=1)
    #	    raise redirect("lite")

# called by Kid template
def test_cancelled(journals): #pragma: no cover
	all_cancelled = 0 
	for j in journals:
		all_cancelled = 1
		if j.cancelled <= 0:
			all_cancelled = 0
			break
	return all_cancelled

