import cherrypy

import turbogears
from turbogears import controllers, expose, validate
from turbogears import identity, validators

from luca.model import conn, notfound, denied
from luca import model
from luca.accountlib import check_chart_mask

from luca.simple_table import simple_table

class charts(simple_table):

# SUBCLASS-SPECIFIC FUNCTIONS AND PARAMETERS ##########################################################
	
    validators = {"id": validators.Int(if_invalid=0), 
        	  "description": validators.UnicodeString(if_invalid=u""),
        	  "mask": validators.UnicodeString(if_invalid=u""),
		  "copyfrom": validators.Int(if_invalid=0)}
				 
    def setup(self):
	self.emptyrow_read = {"id": 0, "description": "", "mask": ""}
	self.emptyrow_write = self.emptyrow_read
	self.empty_intermediate = self.emptyrow_read.copy()
	self.empty_intermediate["copyfrom"] = 0

	# data for automated tests
	self.sample_data = [{"mask": "9.99", "copyfrom": 0}, \
			    {"mask": "9.99.AA", "copyfrom": 0}, \
			    {"mask": "9.99.AA", "copyfrom": 0}, \
			    {"mask": "9.99.XX", "copyfrom": 0}]
	self.invalid_data = [("copyfrom", 444), ("mask", "9.ABC"), ("mask", ""), ("mask", "9.9 9"), ("mask", "9..9"),  ("description", "")]
	self.unique_cols = []
	self.foreign_keys = []
	# TODO: autotest copyfrom in include
	self.test_order = 2
	self.test_flags = ('lite_update_calls_all_rows',)
	self.test_bump = [ ('charts', 'version', 'id'), ('cmos', 'charts_version', 1) ]

    	self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
    	self.msg_notfound = "Chart of accounts not found, probably it has just been deleted"
	self.show_msg_ajax = True
    	self.msg_added = "Chart of accounts %d has been added"
    	self.msg_updated = "Chart of accounts %d has been updated"
    	self.msg_removed = "Chart of accounts %d has been removed sucessfully"
	self.msg_validation = "Order number and description must not be empty"
    	self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
    	self.msg_integrity = "Integrity error: "
    	self.msg_cannotremove = "Chart of accounts can not be removed"
    	self.msg_unique = "check if the order number is unique"
    	self.msg_legend_update = "Charts of accounts: editing chart %s"
    	self.msg_legend_add = "Charts of accounts: new chart"
	self.msg_maskchanged = "You can not change mask after you have accounts in a chart!"

    	self.table = model.charts

    def row_name(self, d_row):
	name = ""
	if d_row["id"] > 0:
		name = u"%s (id %d)" % (d_row["description"], d_row["id"])
	else:
		name = d_row["description"]
	return name

    def data_legend(self, intermediate):
        ''' Returns a legend for the given data'''
        return unicode(intermediate["id"])+u" - "+intermediate["description"]

    def row_to_intermediate_ext(self, row, intermediate):
	intermediate["copyfrom"] = 0

    def form_to_intermediate_ext(self, form_data, d):
   	d["description"] = d["description"][:40].strip()
   	d["mask"] = d["mask"][:20].strip().upper()

    def intermediate_to_form_ext(self, intermediate, form_data):    
	form_data["copyfrom"] = 0

    def intermediate_validate(self, d):
	    if not d["description"]:
		return "Description must not be empty"

	    if d["id"] > 0:
		   row = self.table.get(d["id"])
		   if row.mask != d["mask"] and len(row.accounts) > 0:
		 	return self.msg_maskchanged
		   if d["copyfrom"] > 0:
			return "Can not copy from other chart in update"
	    elif d["copyfrom"] > 0:
		row_from = self.table.get(d["copyfrom"])
		if row_from.mask != d["mask"]:
			return "Mask does not match the chart mask (%s) where accounts would be copied from" % row_from.mask
		    
	    return check_chart_mask(d["mask"])

# INHERITED METHODS ########################################################

    def trigger_update_post(self, row, id, new, form_data):
	    if not new:
		row = self.table.get(id)
		row.bump_version()
	    model.cmos.bump_version("charts")

	    copyfrom = form_data["copyfrom"]

	    if new and copyfrom > 0:
		# copy accounts from existing chart to new
		row_from = self.table.get(copyfrom)
		row_to = self.table.get(id)
		for account in row_from.accounts:
			d_account = {}
			d_account["chart"] = row_to
			for item in "xclass", "stdjournal", "code", "parent", "description", \
				    "smalldesc", "aggregate", "mobile", "inactive":
				d_account[item] = getattr(account, item)
			model.accounts(**d_account)

    def table_version(self, row):
	return model.cmos.get_version("charts")

    def trigger_remove_pre(self, row):
	t = model.orgs.select("chart_id = %d" % row["id"])
	for l in t:
	    return self.msg_integrity + "Chart is in use by organization %d" % l.id
	t = model.accounts.select("chart_id = %d" % row["id"])
	for l in t:
	    return self.msg_integrity + "Chart contains account %s" % (l.code)
	model.cmos.bump_version("charts")
	return ""

# EXPOSED METHODS ########################################################

    @expose(template="luca.templates.charts")
    def index(self, **dummy):
        return self.base_index(orderBy=self.table.q.description)

    @expose(template="luca.templates.charts_lite")
    def lite(self, **dummy):
        return self.base_index(orderBy=self.table.q.description)

    @expose("json")
    @validate(validators=validators)
    def remove(self, id, **garbage):
	    result = self.base_remove(id)
	    return result

    @expose()
    @validate(validators=validators)
    def lite_remove(self, id, **garbage):
	    return self.base_lite_remove(id)

    @expose("json")
    @validate(validators=validators)
    def fetch(self, id, **garbage):
	    return self.base_fetch(id)

    @expose(template="luca.templates.charts_lite_form")
    @validate(validators=validators)
    def lite_edit(self, id, **garbage):
	form_data = self.base_lite_edit(id)
	if id <= 0:
		form_data["rows"] = self.all_rows(orderBy=self.table.q.description)[0]
	else:
		form_data["rows"] = []
	return form_data

    @expose("json")
    @validate(validators=validators)
    def update(self, **form_data):
	    result = self.base_update(form_data)
	    return result

    @expose(template="luca.templates.charts_lite_form")
    @validate(validators=validators)
    def lite_commit(self, **form_data):
	    form_data = self.base_lite_commit(form_data)
	    form_data["rows"] = self.all_rows(orderBy=self.table.q.description)[0]
	    return form_data

