#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import os
import sys

from os.path import *

import pkg_resources
pkg_resources.require("TurboGears")

import turbogears

def start():
    """Start the CherryPy application server."""

    import cherrypy
    cherrypy.lowercase_api = True

    curdir = os.getcwd()

    # first look on the command line for a desired config file,
    # if it's not on the command line, then look for setup.py
    # in the current directory. If it's not there, this script is
    # probably installed
    if len(sys.argv) > 1:
        turbogears.update_config(configfile=sys.argv[1],
            modulename="luca.config")
    elif exists(join(curdir, "setup.py")):
        turbogears.update_config(configfile="dev.cfg",
            modulename="luca.config")
    else:
        try:
            configfile = pkg_resources.resource_filename(
                pkg_resources.Requirement.parse("luca"), "default-prod.cfg")
        except pkg_resources.DistributionNotFound:
            if exists(join(curdir, "prod.cfg")):
                filename = "prod.cfg"
            else:
                print "No configuration found! Aborting..."
                sys.exit(1)

        turbogears.update_config(configfile=configfile,
            modulename="luca.config")

    from luca.controllers import Root
    from luca import model
    import time

    try:
        from luca import bluetooth_server
        blueserver = bluetooth_server.server() # threaded bluetooth server
        blueserver.start()
    except ImportError:
        print "Bluetooth server disabled due to lack of LightBlue library"

    model.touch()  # inits tables
    turbogears.start_server(Root())
    bluetooth_server.running = 0
    time.sleep(2)

if __name__ == '__main__':
    sys.exit(start(sys.argv[1:]))

