import cherrypy

import turbogears
from turbogears import controllers, expose, validate, redirect
from turbogears import identity, validators
from sqlobject import sqlbuilder
from model import permission

from luca.model import conn, notfound, sqlerror, denied
from luca import model, mylib

from luca.threelevel_table import threelevel_table

class journals(threelevel_table):

    validatorz = {"id": validators.Int(if_invalid=0), 
		  "org": validators.Int(if_invalid=0),   # grandmother
		  "batch": validators.Int(if_invalid=0),   # mother
		  "accountDB": validators.Int(if_invalid=0), 
		  "accountCR": validators.Int(if_invalid=0), 
		  "stdjournal": validators.Int(if_invalid=0), 
		  "costcenter": validators.Int(if_invalid=0), 
        	  "day": validators.UnicodeString(if_invalid=u""),
        	  "description": validators.UnicodeString(if_invalid=u""),
		  "value": validators.Int(if_invalid=0),
		  "cancelled": validators.Int(if_invalid=0)}

    validatorz_mother = {"grandmother": validators.Int(if_invalid=0), 
			 "mother": validators.Int(if_invalid=0),
			 "id": validators.Int(if_invalid=0),
		   	 "version": validators.Int(if_invalid=0)}
				 
    def setup(self):
	self.emptyrow_read = {"id": 0, "org": 0, "batch": 0, "account": 0, \
			      "stdjournal": 1, "costcenter": 1, "day": "", "description": "",  "value": 0,
			      "cancelled": 0}
	self.emptyrow_write = self.emptyrow_read.copy()
	self.emptyrow_write["hash"] = ""
	self.emptyrow_write["transa"] = None
	self.empty_intermediate = self.emptyrow_read

	# data for automated tests
	# transactions should be ignored completely and not trigger a failure, hence we try to insert them in some entries
	self.sample_data = [ {"org": 1, "batch": 1, "accountDB": 3, "accountCR": 0, "stdjournal": 1, "costcenter": 1, "day": "2006/01/05", "value": 123, "cancelled": 0}, \
			     {"org": 1, "batch": 1, "accountDB": 0, "accountCR": 4, "stdjournal": 1, "costcenter": 1, "day": "2006/01/05", "value": 123, "cancelled": 0}, \
			     {"org": 1, "batch": 1, "accountDB": 3, "accountCR": 4, "stdjournal": 1, "costcenter": 1, "day": "2006/01/05", "value": 200, "cancelled": 0}, \
			     {"org": 1, "batch": 1, "accountDB": 9, "accountCR":10, "stdjournal": 1, "costcenter": 1, "day": "2006/01/05", "value": 1234, "cancelled": 0}, \
			     {"org": 1, "batch": 1, "accountDB": 4, "accountCR": 0, "stdjournal": 1, "costcenter": 1, "day": "2006/01/10", "value": 3000, "cancelled": 0, "transa": 999}, \
			     {"org": 1, "batch": 1, "accountDB": 0, "accountCR": 3, "stdjournal": 1, "costcenter": 1, "day": "2006/01/10", "value": 3000, "cancelled": 0}, \
			     {"org": 1, "batch": 1, "accountDB": 3, "accountCR": 0, "stdjournal": 1, "costcenter": 1, "day": "2006/01/10", "value": 100, "cancelled": 0, "transa": -1}, \
			     {"org": 1, "batch": 1, "accountDB": 0, "accountCR": 4, "stdjournal": 1, "costcenter": 1, "day": "2006/01/10", "value": 100, "cancelled": 0}, \
			     {"org": 1, "batch": 1, "accountDB": 3, "accountCR": 0, "stdjournal": 1, "costcenter": 1, "day": "2006/01/10", "value": 9999999999, "cancelled": 0}, \
			     {"org": 1, "batch": 1, "accountDB": 0, "accountCR": 4, "stdjournal": 1, "costcenter": 1, "day": "2006/01/10", "value": 9999999999, "cancelled": 0} ]
	self.invalid_data = [ ("description", ""), ("stdjournal", 4444), ("costcenter", 4443),  \
			      ("accountDB", 444), ("accountCR", 444), \
			      ("accountDB", 1), ("accountCR", 2), ("day", "4.44"),  \
			      ("day", "5"), ("value", 0), ("batch", 999), ("org", 999), \
			      ("costcenter", 999), ("stdjournal", 999) ]

	self.unique_cols = []
	self.foreign_keys = [("orgs", 1), ("batches", 1), ("stdjournals", 1), ("costcenters", 1)]

	self.invalid_compound_new = [   dict(accountDB=4, accountCR=555), \
					dict(accountCR=555, accountDB=3), 
				        dict(accountDB=5, accountCR=0) ]
	self.test_order = 11
	self.test_bump = [ ('batches', 'version', 'batch') ]

    	self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
    	self.msg_notfound = "Journal entry not found, probably it has just been deleted"
	self.msg_grandmotherinvalid = "Organization ID invalid (perhaps organization has just been deleted?)"
	self.msg_motherinvalid = "Batch ID invalid (perhaps batch has just been deleted?)"
	self.show_msg_ajax = True
    	self.msg_added = "Journal entry %d has been added"
    	self.msg_updated = "Journal entry %d has been updated"
    	# self.msg_removed = "Journal entry %d has been removed sucessfully"
	self.msg_validation = "Invalid data, please fix"
    	self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
    	self.msg_integrity = "Integrity error: "
    	# self.msg_cannotremove = "Journal entry can not be removed"
    	self.msg_unique = "Journal entry is duplicate (duplicate hash) - try again"
    	self.msg_legend_update = "Journal entries: editing %s"
    	self.msg_legend_add = "Journal entries: new entry"
	self.msg_onlyoneaccount = "Fill either the debit or the credit account, not both"
	self.msg_sameaccount = "Debit and credit accounts must be different"
	self.msg_wrongaccount = "Account does not exist (maybe has just been removed?)"

	self.title = "Journal entries"
	self.title_mask = "Journal entries - %d %s, batch %s"

    	self.table = model.journal
	self.grandmother_table = model.orgs
	self.mother_table = model.batches
	self.grandmother_field_name = "org"
	self.mother_field_name = "batch"

	self.formatters = {"_extra_": self.format_data}

    def test__new_row_fixup(self, row): #pragma: no cover
	del row["account"]
	row["accountDB"] = 0
	row["accountCR"] = 0

    def test__verify_excuse(self, col, orig, fetched, orig_row, fetched_row): #pragma: no cover
	return ((col == "accountDB" and fetched == 0 and orig != 0) and (orig_row["accountDB"] > 0 and orig_row["accountCR"] > 0)) or \
	       ((col == "accountCR" and fetched == 0 and orig != 0) and (orig_row["accountDB"] > 0 and orig_row["accountCR"] > 0))

    def test__should_verify_update(self, row): #pragma: no cover
	if row["accountDB"] > 0 and row["accountCR"] > 0:
		return 0
	return 1

    def test__cancel(self, t, grandmother, mother, id): #pragma: no cover
	t.cargo = res = t.tu.call(self.cancelled, grandmother, mother, id)
	return not res["err"]

    def test__checksum(self, t, grandmother, mother): #pragma: no cover
	t.cargo = res = t.tu.call(self.checksum, grandmother, mother)
	return not res["err"]

    def test__getchart(self, t, grandmother, mother): #pragma: no cover
	t.cargo = res = t.tu.call(self.get_chart, grandmother, mother, -1)
	return not res["err"]

    def test__valid_cancel(self, t): #pragma: no cover
	t.logmask = 0
	if not t.permission_explode("valid cancel", self.test__cancel, t.DONE, t.rows[-2]["org"], t.rows[-2]["batch"], t.rows[-2]["id"]):
		t.fail("Failed to cancel %d" % rows[-2]["id"])
	if not t.permission_explode("valid cancel", self.test__cancel, t.DONE, t.rows[-1]["org"], t.rows[-1]["batch"], t.rows[-1]["id"]):
		t.fail("Failed to cancel %d" % rows[-1]["id"])

    def test__valid_checksum(self, t): #pragma: no cover
	t.logmask = t.DONE
	if not t.permission_explode("valid checksum", self.test__checksum, t.DONE, t.rows[-2]["org"], t.rows[-2]["batch"]):
		t.fail("Failed to checksum")
	db = mylib._unmonetary(t.cargo["total_db"])
	cr = mylib._unmonetary(t.cargo["total_cr"])
	if db <= 0 or cr <= 0 or db > 999999 or cr > 999999:
		t.fail("Unexpected checksum value: %d %d" % (t.cargo["total_db"], t.cargo["total_cr"]))
	
    def test__valid_getchart(self, t): #pragma: no cover
	t.logmask = t.DONE
	if not t.permission_explode("valid get_chart", self.test__getchart, t.DONE, t.rows[0]["org"], t.rows[0]["batch"]):
		t.fail("Failed to fecth accounts")

    def test__run_extra(self, t): #pragma: no cover
	self.test__valid_cancel(t)
	self.test__valid_checksum(t)
	self.test__valid_getchart(t)

    def format_data(self, d):
	d['value_db'] = d['value_cr'] = d["account_full"] = d["costcenter_full"] = ""
	d["description_full"] = d["description"]
	if d["cancelled"] > 0:
		d["cancelled"] = "Cancelled" 
	else:
		d["cancelled"] = "Ok" 

	if int(d["value"]) > 0:
		d["value_cr"] = mylib.monetary(abs(int(d['value'])))
	else:
		d["value_db"] = mylib.monetary(abs(int(d['value'])))
	try:	
		account_row = model.accounts.get(d["account"])
		d["account_full"] = u"%s %s" % (account_row.code, account_row.description)

		if d["stdjournal"] > 1:
			stdjournal_row = model.stdjournals.get(d["stdjournal"])
			d["description_full"] = u"[%d] %s %s" % (d["stdjournal"], stdjournal_row.description, d["description"])
		if d["costcenter"] > 1:
			costcenter_row = model.costcenters.get(d["costcenter"])
			d["costcenter_full"] = u"[%d] %s" % (d["costcenter"], costcenter_row.description)
			d["costcenter_full2"] = u"(%s)" % d["costcenter_full"]
	except sqlerror:  #pragma: no cover
		pass

    def make_title(self, grandmother, mother):
	t = self.title
	if grandmother > 0 and mother > 0:
		try:
			t = self.title_mask % (grandmother, self.grandmother_table.get(grandmother).name, mother)
		except sqlerror:  #pragma: no cover
			pass
	return t

    def row_name(self, d_row):
	org_code = d_row["org"]
	if isinstance(org_code, model.orgs):
		org_code = org_code.id
	batch_code = d_row["batch"]
	if isinstance(batch_code, model.batches):
		batch_code = batch_code.id
	return u"%d::%d::%d" % (org_code, batch_code, d_row["id"])

    def perm_org(self, interm):
	return interm["org"]

    def form_to_intermediate_ext(self, form_data, d):
   	d["description"] = d["description"][:60].strip()
   	d["day"] = d["day"][:12].strip()

    def intermediate_validate(self, d):
	if not d["description"]:
		return "Description must not be empty"
	if d["value"] == 0:
		return "Value must be positive. Don't forget to type it *without* the decimal point/comma"
	day = mylib.ctod(d["day"])
	if not day:
		return "Invalid journal date"

	if not isinstance(d["account"], model.accounts):
		try:
			account_row = model.accounts.get(d["account"])
			if account_row.inactive:
				return "Account %s is inactive and should not be used anymore" % account_row.code
		except notfound, e:
			return self.msg_wrongaccount

	if account_row.aggregate > 0:
		return "Account must NOT be aggregate"

	if not isinstance(d["stdjournal"], model.stdjournals):
		try:
			stdjournal_row = model.stdjournals.get(d["stdjournal"])
			if stdjournal_row.inactive:
				return "Standard journal is inactive and should not be used anymore"
		except notfound, e:
			return "Standard journal not found (maybe just deleted?)"
	
	if not isinstance(d["costcenter"], model.costcenters):
		try:
			costcenter_row = model.costcenters.get(d["costcenter"])
			if costcenter_row.inactive:
				return "Cost center is inactive and should not be used anymore"
		except notfound, e:
			return "Cost center not found (maybe just deleted?)"
	
	if not isinstance(d["org"], model.orgs):
		try:
			org_row = model.orgs.get(d["org"])
		except notfound, e:
			return "Parent organization not found (maybe just deleted?)"
	else:
		org_row = d["org"]

	if account_row.chart != org_row.chart:
		return "Account is not related to organization's chart of accounts (should not happen -- call support!)"

	if not isinstance(d["batch"], model.batches):
		try:
			batch_row = model.batches.get(d["batch"])
		except notfound, e:
			return "Parent batch not found (maybe just deleted?)"
	else:
		batch_row = d["batch"]

	if batch_row.org != org_row:
		return "Batch is not related with chosen organization (should not happen -- call support!)"
	if batch_row.closed > 0:
		return "Batch is closed. If you need to change journals, reopen the batch first."
	if day < batch_row.date1 or day > batch_row.date2:
		return "Journal date is outside batch period."
	if day < org_row.active_d1 or day > org_row.active_d2:
		return "Journal date is outside organization's active period (should not happen -- call support!)."

	if d["id"] > 0:
		row = self.table.get(d["id"])
		if row.batch.id != batch_row.id:
			return "Tried to change journal's batch (NOT ALLOWED!)"
		if row.org.id != org_row.id:
			return "Tried to change journal's organization (NOT ALLOWED!)"
		if row.transa != None and row.transa > 0:
			return "This entry was automatically generated (transaction %d), it can not be updated from here." % row.transa
	if "transa" in d: #pragma: no cover
		# paranoid check
		return "You can not insert journal entries with transaction via this page!"

	return ""

    def intermediate_to_row_ext(self, intermediate, row):
	row["day"] = mylib.ctod(row["day"])
	row["transa"] = None
	if row["id"] <= 0:
		row["hash"] = model.journal_hash(row)
	else:
		j = self.table.get(row["id"])
		# TODO: optimize out the following attributions
		row["hash"] = j.hash
		row["transa"] = j.transa

    def row_to_intermediate_ext(self, row, intermediate):
	    '''Data dictionary manipulation just before editing an existing row'''
	    # Reduces sub-rows to their IDs
	    if isinstance(intermediate["org"], model.orgs):
		intermediate["org"] = intermediate["org"].id
	    if isinstance(intermediate["batch"], model.batches):
		intermediate["batch"] = intermediate["batch"].id
	    if isinstance(intermediate["account"], model.accounts):
		intermediate["account"] = intermediate["account"].id
	    if isinstance(intermediate["stdjournal"], model.stdjournals):
		intermediate["stdjournal"] = intermediate["stdjournal"].id
	    if isinstance(intermediate["costcenter"], model.costcenters):
		intermediate["costcenter"] = intermediate["costcenter"].id
	    intermediate["day"] = mylib.dtoc(intermediate["day"])
	    intermediate["value"] = int(intermediate["value"])

    def intermediate_to_form_ext(self, intermediate, form_data):
	if intermediate["id"] <= 0:
		# new row, get journal-specific defaults
		# Null = -1 since a legitimate entry can have been single leg (one account = 0)
		form_data["accountDB"] = mylib.getdefault(self, "accountDB", -1, -1)
		form_data["accountCR"] = mylib.getdefault(self, "accountCR", -1, -1)
		# if value is -1, correct to 0
		if form_data["accountDB"] < 0:
			form_data["accountDB"] = 0
		if form_data["accountCR"] < 0:
			form_data["accountCR"] = 0
	else:
		# existing row, always single leg
		form_data["accountDB"] = form_data["accountCR"] = 0
		if form_data["value"] < 0:
			form_data["accountDB"] = form_data["account"]
		else:
			form_data["accountCR"] = form_data["account"]

	form_data["value"] = abs(form_data["value"])

    def data_legend(self, d):
	    return u"%d :: %d :: %d" % (d["org"], d["batch"], d["id"])

    def trigger_update_post(self, row, id, new, form_data):
	if (not new) or (form_data["accountDB"] == 0) or (form_data["accountCR"] == 0) or (form_data["value"] < 0):
		# print "%d %d %d %d" % (new, form_data["accountDB"], form_data["accountCR"], form_data["value"])
		mother_row = row["batch"]
		if not isinstance(mother_row, self.mother_table):
			mother_row = self.mother_table.get(row["batch"])
		mother_row.bump_version()

    # def trigger_remove_post(self, row):
    #    mother_row = row["batch"]
    #    if not isinstance(mother_row, self.mother_table):
    # 	     mother_row = self.mother_table.get(row["batch"])
    #    mother_row.bump_version()

    def table_version(self, row):
	version = -1
	if "batch" in row and row["batch"] > 0:
	    version = self.mother_table.get(row["batch"]).version
	elif "mother" in row and row["mother"] > 0:
	    version = self.mother_table.get(row["mother"]).version
	return version

# INHERITED METHODS ########################################################

    def base_remove(self, **garbage): #pragma: no cover
	# paranoid check
	raise Exception("Can never remove a journal entry!")

    def new_row_recoil(self, row_data):
	row_data["value"] = 0

    def calculate_checksum(self, grandmother, mother):
	err = ""
	total_db = total_cr = 0
	try:
		grant = permission(self, "read", self.perm_org({"org": grandmother}), permission.MASK_DONE, "Checksum %d" % mother)
		batch_row = model.batches.get(mother)
		journals = self.table.select(sqlbuilder.AND(sqlbuilder.SQLConstant("org_id = %d" % grandmother), \
							    self.table.q.cancelled <= 0, \
							    sqlbuilder.SQLConstant("batch_id = %d" % mother)) )
		for journal in journals:
			if int(journal.value) > 0:
				total_cr += abs(int(journal.value))
			else:
				total_db += abs(int(journal.value))
	except denied, e:
		err = e.args[0]
	except sqlerror, e:  #pragma: no cover
		err = self.sqlerr(e) 

	return {"total_db": mylib.monetary(total_db), "total_cr": mylib.monetary(total_cr), "err": err}

    def date_to_class(self, date):
	return "da"+str(date).replace("-", "_")

    def get_index_rows(self, grandmother, mother):
	rows = []
	softerr = ""
	err = ""
	closed = 0
	total_db = total_cr = 0
	if grandmother <= 0 or mother <= 0:
		return (rows, err, softerr, {"total_db": "", "total_cr": "", "batch_open": False})

	try:
		batch_row = model.batches.get(mother)
		closed = batch_row.closed
		journals = self.table.select(sqlbuilder.AND(sqlbuilder.SQLConstant("org_id = %d" % grandmother), \
							    sqlbuilder.SQLConstant("batch_id = %d" % mother)), \
					     orderBy=(self.table.q.day, self.table.q.id))
		rows = []
		total_day = 0
		unbalanced = False
		last_day = batch_row.date1
		last_day_report = ""
		for journal in journals:
			d = {}
			rows.append(d)
			d["id"] = journal.id
			d["cancelled"] = journal.cancelled
			d["day"] = journal.day
			d["account_full"] = journal.account.code + u" " + journal.account.description
			if journal.stdjournal.id != 1:
				d["description_full"] = u"["+str(journal.stdjournal.id)+"] "+journal.stdjournal.description + u" " + journal.description
			else:
				d["description_full"] = journal.description
			if journal.costcenter.id != 1:
				d["costcenter_full"] = u"["+str(journal.costcenter.id)+"] "+journal.costcenter.description
			else:
				d["costcenter_full"] = None
			d["value_db"] = d["value_cr"] = None
			# d["value"] = journal.value

			if last_day_report == journal.day:
				d["day_dtoc"] = ""
				d["collapse_class"] = "invisible "+self.date_to_class(journal.day)
				d["code_class"] = ""
			else:
				d["day_dtoc"] = mylib.dtoc(journal.day)
				last_day_report = journal.day
				d["collapse_class"] = "collapsed"
				d["code_class"] = self.date_to_class(journal.day)

			if last_day != journal.day:
				if total_day != 0:
					unbalanced = True
					# paints the unbalanced day
					for row in rows:
						if row["day"] == last_day:
							row["unbalanced"] = True
				total_day = 0
				last_day = journal.day

			if int(journal.value) > 0:
				d["value_cr"] = mylib.monetary(abs(int(journal.value)))
				total_cr += abs(int(journal.value))
			else:
				d["value_db"] = mylib.monetary(abs(int(journal.value)))
				total_db += abs(int(journal.value))
			total_day += int(journal.value)
			d["unbalanced"] = False
		else:
			if total_day != 0:
				unbalanced = True
				# paints the unbalanced day
				for row in rows:
					if row["day"] == last_day:
						row["unbalanced"] = True

		if total_db != total_cr:
			softerr = "Batch is not balanced" 
		elif batch_row.total > 0: 
			if int(batch_row.total) > total_cr:
				softerr = "Journal entries sum %s more than batch total" %  \
						mylib.monetary(total_cr - int(batch_row.total))
			elif int(batch_row.total) < total_cr:
				softerr = "Journal entries sum %s less than batch total" %  \
						mylib.monetary(int(batch_row.total) - total_cr)
		elif unbalanced:
			softerr = "Journal entries have inter-day unbalances, affected days are color-marked in list"

	except sqlerror, e:  #pragma: no cover
		err = self.sqlerr(e) 

	return (rows, err, softerr, {"total_db": mylib.monetary(total_db), \
				     "total_cr": mylib.monetary(total_cr), \
				     "batch_open": not closed})

    def mother_check_ext(self, grandmother, mother, mother_row):
	return mother_row.date1 >= mother_row.org.active_d1 and mother_row.date2 <= mother_row.org.active_d2

    def auxiliary_form_data(self, form_data):
	err = ""
	form_data["cancelledlist"] = [{"id": 0, "description": "Ok"}, {"id": 1, "description": "Cancelled"}]
	form_data["accounts"] = [{"id": 0, "description": "---"}]
	form_data["mothers"] = [{"id": 0, "description": "---"}]
	form_data["grandmothers"] = [{"id": 0, "description": "---"}]
	for row in model.orgs.select(orderBy=model.orgs.q.name):
		form_data["grandmothers"].append({"id": row.id, "description": "%d - %s" % (row.id, row.name)})
	if form_data[self.grandmother_field_name] > 0:
		org_row = model.orgs.get(form_data[self.grandmother_field_name])
		for row in model.batches.select(sqlbuilder.AND( \
						sqlbuilder.SQLConstant("org_id = %d" % org_row.id),
						model.batches.q.date1 >= org_row.active_d1,
						model.batches.q.date2 <= org_row.active_d2),
						orderBy=model.batches.q.description):
			form_data["mothers"].append({"id": row.id, "description": u"%d - %s%s" % \
						(row.id, row.closed and "(CLOSED) " or "", row.description)})
		for row in model.accounts.select(sqlbuilder.AND( \
						 sqlbuilder.SQLConstant("chart_id = %d" % org_row.chart.id)),
						 orderBy=model.accounts.q.code):
			if row.inactive:
				desc = row.code+u" (inactive) "+row.description
			else:
				desc = row.code+u" "+row.description
			form_data["accounts"].append({"id": row.id, "description": desc})
	return err

    def journal_reduce(self, form_data):
	form_data["value"] = abs(form_data["value"])
	if form_data["accountDB"] > 0:
		form_data["account"] = form_data["accountDB"]
		form_data["value"] = -form_data["value"]
	else:
		form_data["account"] = form_data["accountCR"]

    def journal_update(self, form_data):
	f = [form_data.copy()]
	mylib.setdefault(self, "accountDB", form_data["accountDB"], -1)
	mylib.setdefault(self, "accountCR", form_data["accountCR"], -1)
	form_data["value"] = abs(form_data["value"])

	if form_data["accountDB"] > 0 and form_data["accountCR"] > 0:
		if form_data["id"] > 0:
			return {"err": self.msg_onlyoneaccount, "id": form_data["id"], "new": 0, "msg": ""}
		if form_data["accountDB"] == form_data["accountCR"]:
			return {"err": self.msg_sameaccount, "id": form_data["id"], "new": 0, "msg": ""}
		f.append(form_data.copy())
		f[0]["accountCR"] = -2
		f[1]["accountDB"] = -2
		self.journal_reduce(f[0])
		self.journal_reduce(f[1])

		try:
			# need to pre-check things here because if only one account is wrong
			# the other would be inserted anyway

			# WARNING: CHANGING THE ORDER OF COMMIT OF THE TWO JOURNALS BREAKS
			#	THE VERSION BUMPING, VERSION SHOULD BE BUMPED AS THE *FIRST* 
			#	JOURNAL IS COMMITTED (right now it is the Debit journal)
			#	SINCE JAVASCRIPT CLIENTS ONLY ANALYZE THE FIRST JOURNAL
			#	TO GET THE NEWEST VERSION NUMBER.

			i = self.form_to_intermediate(f[0])	
			j = self.form_to_intermediate(f[1])
			err1 = self.intermediate_validate(i)
			err2 = self.intermediate_validate(j)
			if err1 or err2:
				return {"err": err1 or err2, "id": form_data["id"], "new": 0, "msg": ""}
		except sqlerror, e:  #pragma: no cover
			err = self.sqlerr(e) 
			return {"err": err, "id": form_data["id"], "new": 0, "msg": ""}
		
		result = self.base_update(f[0])
		if not result["err"]:
			result2 = self.base_update(f[1])
			result["row2_formatted"] = result2["row_formatted"]
			result["id2"] = result2["id"]
	else:
		self.journal_reduce(f[0])
		result = self.base_update(f[0])

	return result

    def journal_update_lite(self, form_data):
	is_streamlined = self.streamlined_add and form_data["id"] <= 0

	result = self.journal_update(form_data)

	if not result["err"]:
		if is_streamlined:
			self.flash_at_edit(result["msg"], err=0)
			if model._testing_:
				return {"err": ""}
			raise redirect(turbogears.url("lite_edit", id=0, \
					grandmother=form_data[self.grandmother_field_name],  \
					mother=form_data[self.mother_field_name])) 

		self.flash_at_index(result["msg"], err=0)
		if model._testing_:
			return {"err": ""}
		raise redirect(turbogears.url("lite", grandmother=form_data[self.grandmother_field_name], \
                                        mother=form_data[self.mother_field_name])) 

	# error happened: send back all received data + the error message
	form_data = self.intermediate_to_form(self.form_to_intermediate(form_data))
	form_data.update(result)

	err = ""
	try:
		err = self.auxiliary_form_data(form_data)
	except sqlerror, e:  #pragma: no cover
		err = "SQL error inside auxiliary_form_data" 
	if err:
		form_data["err"] = err

	return form_data

    def handle_reopen_batch(self, org, batch):
	form_data = cherrypy.root.batches.toogle_closed(org, batch)
        if form_data["err"]:
		self.flash_at_index("Could not update status: "+form_data["err"], err=1)


    def handle_get_chart(self, org, batch, client_version):
	l = [[0, u"---", 0, 1, 0]]
	err = ""
	version = 0
	try:
		grant = permission(self, "read", self.perm_org({"org": org}), permission.MASK_DONE, "Get chart AJAX")
		org_row = model.orgs.get(org)
		version = org_row.chart.version
		if client_version != version:
			for row in model.accounts.select(sqlbuilder.AND( \
							 sqlbuilder.SQLConstant("chart_id = %d" % org_row.chart.id)),
							 orderBy=model.accounts.q.code):
				if row.inactive:
					desc = row.code+u" (inactive) "+row.description
				else:
					desc = row.code+u" "+row.description
				l.append([row.id, desc, row.inactive, row.stdjournal.id, row.xclass.is_income])
	except denied, e:
		err = e.args[0]
	except sqlerror:  #pragma: no cover
		err = "Error while retrieving accounts: " + self.sqlerr(e) 
	return {"chart_list": l, "err": err, "version": version }

# EXPOSED METHODS ########################################################

    @expose(template="luca.templates.journals")
    @validate(validators=validatorz_mother)
    def index(self, grandmother=0, mother=0, **dummy):
        d = self.base_index(False, grandmother, mother)
	if "batch_open" not in d:
		d["batch_open"] = 1
	if "total_db" not in d:
		d["total_db"] = ""
	if "total_cr" not in d:
		d["total_cr"] = ""
	return d

    @expose(template="luca.templates.journals_lite")
    @validate(validators=validatorz_mother)
    def lite(self, grandmother=0, mother=0, **dummy):
        d = self.base_index(True, grandmother, mother)
	if "batch_open" not in d:
		d["batch_open"] = 1
	if "total_db" not in d:
		d["total_db"] = ""
	if "total_cr" not in d:
		d["total_cr"] = ""
	return d

    @expose("json")
    @validate(validators=validatorz_mother)
    def fetch(self, grandmother, mother, id, **garbage):
	    return self.base_fetch(grandmother, mother, id)

    @expose("json")
    @validate(validators=validatorz_mother)
    def get_chart(self, grandmother, mother, version, **garbage):
	    return self.handle_get_chart(grandmother, mother, version)

    @expose()
    @validate(validators=validatorz_mother)
    def reopen_batch(self, grandmother, mother, **garbage):
	   self.handle_reopen_batch(grandmother, mother)
	   raise redirect(turbogears.url("./", grandmother=grandmother, mother=mother))

    @expose()
    @validate(validators=validatorz_mother)
    def reopen_batch_lite(self, grandmother, mother, **garbage):
	   self.handle_reopen_batch(grandmother, mother)
	   raise redirect(turbogears.url("lite", grandmother=grandmother, mother=mother))

    @expose(template="luca.templates.journals_lite_form")
    @validate(validators=validatorz_mother)
    def lite_edit(self, grandmother, mother, id, **garbage):
	   d = self.base_lite_edit(grandmother, mother, id)
	   return d

    @expose("json")
    @validate(validators=validatorz)
    def update(self, **form_data):
	    return self.journal_update(form_data)

    @expose(template="luca.templates.journals_lite_form")
    @validate(validators=validatorz)
    def lite_commit(self,  **form_data):
	    return self.journal_update_lite(form_data)

    @expose("json")
    @validate(validators=validatorz_mother)
    def checksum(self, grandmother, mother, **garbage):
	d = self.calculate_checksum(grandmother, mother)
	return d

    def toogle_cancelled(self, grandmother, mother, id):
	    form_data = self.base_fetch(grandmother, mother, id)
	    if form_data["err"]:
		    return form_data
	    if form_data["cancelled"]:
	    	form_data["cancelled"] = 0
	    else:
	    	form_data["cancelled"] = 1
	    form_data["err"] = self.journal_update(form_data)["err"]
	    return form_data

    @expose("json")
    @validate(validators=validatorz_mother)
    def cancelled(self, grandmother, mother, id, **garbage):
	return self.toogle_cancelled(grandmother, mother, id)

