# -*- coding: utf-8 -*-

from luca.model import notfound, sqlerror, log, function_points_nowild, PermissionError
import luca.controllers
from sqlobject import *
import time
import datetime
from luca import model
import random
from turbogears.identity import current

######################################################### NUMBER FORMATTING

def _unmonetary(num_as_string):
	num_as_string = num_as_string.replace('.', '')
	num_as_string = num_as_string.replace(',', '')
	num_as_string = num_as_string.replace(' ', '')
	return int(num_as_string)

def monetary(numberX100):
	if len(current.user.locale) >= 2:
		thousand_separator = current.user.locale[0]
		decimal_separator = current.user.locale[1]
	else: #pragma: no cover
		# paranoid
		thousand_separator = u"."
		decimal_separator = u","

	signal = numberX100 < 0 and u"-" or u""
	numberX100 = abs(numberX100)
	cents = numberX100 % 100
	sinteger = u"%d" % ((numberX100 - cents) / 100)
	sinteger2 = ""
	while len(sinteger) > 3:
		sinteger2 = thousand_separator + sinteger[-3:] + sinteger2
		sinteger = sinteger[:-3]
	return signal + sinteger + sinteger2 + decimal_separator + u"%02d" % cents

def monetary_dbcr(numberX100):
	return monetary(abs(numberX100)) + (numberX100 > 0 and " CR" or " DB")

def monetary_dbcr_color(numberX100, normal_balance):
	txt = monetary_dbcr(numberX100)
	css = "positive"
	if (numberX100 < 0 and normal_balance <= 0) or (numberX100 > 0 and normal_balance > 0):
		css = "negative"
	return (txt, css)

def monetary_balance(numberX100):
	number = monetary(abs(numberX100))
	css = "positive"
	if numberX100 == 0:
		number = u""
	elif numberX100 < 0:
		number = u"(" + number + u")"
		css = "negative"
	return (number, css)

# def monetary_balance_withzero(numberX100):
# 	number = monetary(abs(numberX100))
# 	css = "positive"
# 	if numberX100 < 0:
# 		number = u"(" + number + u")"
# 		css = "negative"
#	return (number, css)
	
############################## DATE CHECKING

date_formats = {u"Y/m/d": [u"%(Y)04d/%(m)02d/%(d)02d", u"ymd", u"321"], \
		u"y/m/d": [u"%(y)02d/%(m)02d/%(d)02d", u"ymd", u"321"], \
		u"d/m/Y": [u"%(d)02d/%(m)02d/%(Y)04d", u"dmy", u"123"], \
		u"d/m/y": [u"%(d)02d/%(m)02d/%(y)02d", u"dmy", u"123"], \
		u"m/d/Y": [u"%(m)02d/%(d)02d/%(Y)04d", u"mdy", u"213"], \
		u"m/d/y": [u"%(m)02d/%(d)02d/%(y)02d", u"mdy", u"213"] }

thousand_separators = "., "
decimal_separators  = ",. "

def timestamp():
	day = datetime.date.today()
	tm = datetime.datetime.now()
	s = dtoc(day)+" %02d:%02d:%02d" % (tm.hour, tm.minute, tm.second)
	return s

def dtoc(d):
	''' Returns a locale-aware string of a ISO SQL date or a datetime.date object '''
	try:
		locale_date = current.user.locale[2:7]
		date_mask = date_formats[locale_date][0]
	except KeyError:
		date_mask = date_formats["Y/m/d"][0]

	if d is None:
		d = "1970-01-01"
	elif type(d) not in (str, unicode):
		# datetime.date
		d = str(d)
	dd = {}
	dd["Y"], dd["m"], dd["d"] = [int(x) for x in d.split("-")]
	dd["y"] = dd["Y"] % 100
	return date_mask % dd

def _check_date(year, month, day):
	if month > 12 or month < 1:
		return False

	months = [None, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
	leap = 0

	if month == 2:
		max_day = 28
		if (year % 4 == 0) and ((year % 100 != 0) or (year % 400 == 0)):
			leap = 1
			months[2] = 29

	if day > months[month] or day < 1:
		return False

	if year < 0 or year > 3000:
		return False
	
	return True

def ctod(sdate):
	'''Checks a locale-aware data and returns a ISO SQL date if it is correct.'''

	sdate = sdate.replace(u' ', u'')
	sdate = sdate.replace(u'-', u'/')
	sdate = sdate.replace(u'.', u'/')
	sdate = sdate.replace(u',', u'/')

	dsplit = sdate.split(u'/')

	try:
		dsplit = [int(x) for x in dsplit]
	except ValueError:
		return None

	if len(dsplit) <= 0 or len(dsplit) >= 4:
		return None

	try:
		locale_date = current.user.locale[2:7]
		date_ruler = date_formats[locale_date][1]
	except KeyError:
		date_ruler = date_formats[u"Y/m/d"][1]

	if len(dsplit) == 1:
		# lacks month & year
		yN, mN, dN, x, x, x, x, x, x = time.localtime(time.time())
		y, m, d = ( yN, mN, dsplit[0] )
	elif len(dsplit) == 2:
		# lacks year
		yN, mN, dN, x, x, x, x, x, x = time.localtime(time.time())
		# let's see which one (day or month) comes first
		# (I think this code below is ugly. But if you don't like it, FIX IT and submit a patch :)
		im, id = (0, 1)
		if date_ruler.index("m") > date_ruler.index("d"):
			im, id = (1, 0)
		y, m, d = ( yN, dsplit[im], dsplit[id] )
	else:
		y, m, d = ( dsplit[date_ruler.index("y")], dsplit[date_ruler.index("m")], dsplit[date_ruler.index("d")] )

	if y < 50:
		y += 2000
	elif y < 100:
		y += 1900

	if not _check_date(y, m, d):
		return None
	
	# makes date with 00:00:01AM time
	return datetime.date(y, m, d)

################## DEFAULT VALUE HANDLING

def getdefault(client, name, value, nil):
	if value == nil:
		x = model.session()
		try:
			# try client-specific default
			value = x[client.__class__.__name__ + name]
		except KeyError:
			try:
				# try more generic default
				value = x[name]
			except KeyError:
				pass
	return value

def setdefault(client, name, value, nil):
	if value != nil:
		x = model.session()
		x[client.__class__.__name__ + name] = value
		try:
			generic = x[name]
		except KeyError:
			generic = nil

		if generic == nil:
			# generic default is set only if it is nil
			x[name] = value

def random_string(l):
	# makes sure strings *will* be problematic if there are problems when handling Unicode or special characters
	s = u"".join([ random.choice(u" abcáãõÁÉ²³ABCE!@#$%*()[]{};΅:\"`~\\\'|,./<>?☤☥☹☺☔☃♠♡♢♣") for x in range(0, l) ])
	return s.strip()

