from turbogears import controllers
from turbogears import identity, validators, expose, validate
from sqlobject import sqlbuilder
import simplejson
from luca import model, mylib
from luca.model import ModelError
import datetime
import time
from luca.model import sqlerror, notfound

class remote_mobile(controllers.Controller):

    def __init__(self, turbogears_log):
       	super(remote_mobile, self).__init__()
	self.test_flags = ()
	self.vips = []
	self.org = 0
	# self.setup()
	self.methods = {"helo": self.helo, "getaccounts": self.sendaccounts, "getstdjournals": self.sendstdjournals, \
			"sendjournal": self.getjournal, "confirmjournal": self.confirmjournal}

    def sqlerr(self, exception):  #pragma: no cover
	self.log.error("%s SQL error %s", self.__class__.__name__, str(exception.args))
	return self.msg_unexpected

    def perm_org(self, interm):
	return 0 #pragma: no cover

    @expose("json")
    def index(self, blob, **garbage):
	return self.handle(blob)

    def handle(self, rawdata):
	try:
		data = simplejson.loads(rawdata)
	except ValueError, e:
		return {"error": "Invalid JSON request"}

	try:
		req = data["request"].lower()
	
		if self.org <= 0 and req != "helo":
			raise ModelError("Request sent before HELO")

		try:
			method = self.methods[req]
		except KeyError:
			raise ModelError("Unknown request '%s'" % req)

		answer = method(data)

	except ModelError, e:
		answer = {"error": str(e.args[0])}
	except sqlerror, e:
		answer = {"error": str(e.args)}

	assert type(answer) is dict
	return answer

    def helo(self, data):
	if not data.has_key("org"):
		raise ModelError("HELO has no ORG id")
	self.org = data["org"]
	self.chk_org(self.org)
	return { "response": "helo", "accounts_version": self.accounts_version(self.org), \
			 "stdjournals_version": self.stdjournals_version() }

    def sendaccounts(self, data):
	return { "response": "getaccounts", "payload": self.get_accounts(self.org) }

    def sendstdjournals(self, data):
	return { "response": "getstdjournals", "payload": self.get_stdjournals() }

    def getjournal(self, data):
	journal_batch = data["payload"]
	# datasource may throw exception
	self.journal_update(self.org, journal_batch)
	return { "response": "sendjournal" }

    def confirmjournal(self, data):
	return { "response": "confirmjournal", "exists": self.journal_confirm(self.org, data["key"]) }

    def journalentry_chkdata(self, data):
	try:
		if data["type"] != "journalentry":
			raise ModelError("Data record is corrupted")

		if (not isinstance(data["key"], unicode)) or \
		   (not isinstance(data["account"], unicode)) or \
		   (not isinstance(data["description"], unicode)) or \
		   (not isinstance(data["stdjournal"], unicode)):
			raise ModelError("Some unicode field of wrong type")
		
		try:
			# here we need to resort to a polymorphic test
			x = data["date"] + 1
			x = data["value"] + 1
		except TypeError:
			raise ModelError("Date/Value is not a number.")
	except KeyError, e:
		raise ModelError("Data record lacks key: %s ", str(e.args))

    def journal_confirm(self, org, k):
	c = model.journal.select(sqlbuilder.AND(model.journal.q.hash == k, sqlbuilder.SQLConstant("org_id = %d" % org))).count()
	return c and 1 or 0
	
    def __journal_sort(self, x, y):
	r = cmp(x["date"], y["date"])
	if r == 0:
		r = cmp(x["timestamp"], y["timestamp"])
	return r

    def journal_update(self, org, d):
	datalist = []

	for key in d.keys():
		data = d[key]
		datalist.append(data)

	datalist.sort(lambda x, y: self.__journal_sort(x, y))

	virgin = True
	date1 = time.mktime((2037, 1, 2, 0, 0, 1, 0, 0, 0)) 
	date2 = time.mktime((1970, 1, 1, 0, 0, 1, 0, 0, 0)) 
	total = 0

	for data in datalist:
		self.journalentry_chkdata(data)
		if self.journal_confirm(org, data['key']):
			# journal entry is already in the database!
			continue

		if data["date"] < date1:
			date1 = data["date"]
		if data["date"] > date2:
			date2 = data["date"]

		if data["value"] > 0:
			# sums only the credits
			total += data["value"]

	if not total:
		return

	org_row = model.orgs.get(org)

	if datetime.date.fromtimestamp(date1) < org_row.active_d1 or datetime.date.fromtimestamp(date2) > org_row.active_d2:
		raise ModelError("Journal entries outside the active period")

	batch = model.batches(org=org_row, date1=datetime.date.fromtimestamp(date1), \
					   date2=datetime.date.fromtimestamp(date2), \
					   description=u"Entries from mobile", total=total, closed=1, adjust=0)

	for data in datalist:
		if self.journal_confirm(org, data['key']):
			# journal entry is already in the database!
			continue
		accounts = model.accounts.select(sqlbuilder.AND(sqlbuilder.SQLConstant("chart_id = %d" % org_row.chart.id), \
								model.accounts.q.code == data['account']))
		if not accounts.count():
			raise ModelError("Account %s does not exist" % data["account"])
		for a in accounts:
			account = a

		model.journal(org=org_row, day=datetime.date.fromtimestamp(data['date']), stdjournal=int(data['stdjournal']), \
				costcenter=1, description=data['description'], value=data['value'], batch=batch, account=account, \
				hash=data['key'], cancelled=0, transa=None)

	org_row.bump_version()

    def accounts_version(self, org):
	return model.orgs.get(org).chart.version

    def stdjournals_version(self):
	return model.cmos.get(1).stdjournals_version

    def get_accounts(self, org):
	d = {}
	for row in model.orgs.get(org).chart.accounts:
		if row.mobile <= 0 or row.aggregate > 0 or row.inactive > 0:
			continue
		desc = row.smalldesc.strip() or row.description
		d[row.code] = {"type": "account", "key": row.code, "description": desc}
	return d

    def get_stdjournals(self):
	li = model.stdjournals.select(model.stdjournals.q.mobile > 0)
	d = {}
	for row in model.stdjournals.select(model.stdjournals.q.mobile > 0):
		desc = row.smalldesc.strip() or row.description
		d["%d" % row.id] = { "type": "stdjournal", "key": "%d" % row.id, "description": desc }
	return d

    def chk_org(self, org):
	try:
		model.orgs.get(org)
	except model.notfound:
		raise ModelError("Organization %d does not exist" % org)
	return 1

