// the first control must be the ID, which is hidden
var controls = ["id", "org", "batch", "accountDB", "accountCR", "stdjournal", "costcenter", "day", "description", "value", "cancelled"];
var inplace = ["id", "account_full", "description_full", "value_db", "value_cr", "day", "cancelled"];
var comm_error = "Could not communicate from server. Try again in a few moments.";
var new_row_legend = "New journal entry";
var edit_legend = "Editing journal entry ";
var remove_dialog = "Do you really want to delete the journal entry?";
var fetch_remote_error = "Could not get journal entry from server";
var cancelled_yes = "Cancelled";
var cancelled_no = "Ok";
var grandmother_field = "org";  
var mother_field = "batch";  
var chart_version = -1;
var chart_list = []
var ajax_chart_pending = false;
var ajax_checksum_pending = false;
var accounts_dirty = true;
var edit_gotdata_saved = null;
var filter_inactive_accounts = false;
var jkeys_chart = null;
var jvalues_chart = null;


function ajax_setup_extend()
{
	// extends tables' ajax_setup()
	connect(document.motherselect.grandmother, "onchange", ajax_grandmother_selected);
	connect(document.motherselect.mother, "onchange", ajax_mother_selected);
	connect(document.f.accountDB_filter, "onkeyup", ajax_dbfilter);
	connect(document.f.accountDB_filter, "onblur", ajax_dbfilter);
	connect(document.f.accountCR_filter, "onkeyup", ajax_crfilter);
	connect(document.f.accountCR_filter, "onblur", ajax_crfilter);
	connect(document.f.accountDB, "onchange", ajax_dbchange);
	connect(document.f.accountCR, "onchange", ajax_crchange);
	streamlined_add = true;
	first_visible_field = 3;
}

function ajax_grandmother_selected(e) {
	document.f[grandmother_field].value = document.motherselect.grandmother.value;
	ajax_refresh_page();
}

function ajax_mother_selected(e) {
	document.f[mother_field].value = document.motherselect.mother.value;
	ajax_refresh_page();
}

function ajax_refresh_page_override()
{
	location.href = "./?grandmother="+ document.f[grandmother_field].value + "&mother=" + document.f[mother_field].value;
}

function ajax_edit_grandmother(grandmother_id, mother_id, id)
{
	// replaces ajax_edit() completely, since it has more parameters

	if (ajax_pending) {
		return;
	}

	if (id > 0) {
		// edit existing journal entry: guarantee that all known accounts will be in place
		if (filter_inactive_accounts) {
			filter_inactive_accounts = false;
			accounts_dirty = true;
		}
		if (document.f.accountDB_filter.value.length > 0) {
			document.f.accountDB_filter.value = "";
			accounts_dirty = true;
		}
		if (document.f.accountCR_filter.value.length > 0) {
			document.f.accountCR_filter.value = "";
			accounts_dirty = true;
		}
	} else {
		// new journal entry: only active accounts
		if (! filter_inactive_accounts) {
			filter_inactive_accounts = true;
			accounts_dirty = true;
		}
	}

	ajax_unhide();
	ajax_throbber();

	var jkeys = ["grandmother", "mother", "id"];
	var jvalues = [grandmother_id, mother_id, id];
	var addr = "fetch?" + queryString(jkeys, jvalues);

	var ajax = loadJSONDoc(addr);
	ajax.addCallbacks(ajax_edit_gotdata_pre, ajax_error_hide);

	ajax_pending = true;

	// stored to be called after this JSON transaction ends
	// serialize transactions avoids problems with sqlite, even though can increase latency
	jkeys_chart = ["grandmother", "mother", "version"];
	jvalues_chart = [grandmother_id, mother_id, chart_version];
	
	_is_streamlined = streamlined_add && id <= 0;
}

function ajax_edit_gotdata_pre(rres)
{
	edit_gotdata_saved = rres;

	if (! ajax_chart_pending) {
		ajax_chart_pending = true;
		var addr = "get_chart?" + queryString(jkeys_chart, jvalues_chart);
		var ajax2 = loadJSONDoc(addr);
		ajax2.addCallbacks(ajax_chart_gotdata, ajax_chart_error);
	}
}

function ajax_new_override(e)
{
	if (parseInt(document.f[grandmother_field].value) > 0 && parseInt(document.f[mother_field].value) > 0) {
		ajax_edit_grandmother(document.f[grandmother_field].value, document.f[mother_field].value, -1);
	} 
}

function ajax_chart_gotdata(rres)
{
	ajax_chart_pending = false;

	ajax_throbber_end();

	if (rres.err != "") {
		alert(fetch_remote_error);
		return;
	}

	if (rres.version != chart_version) {
		chart_version = rres.version;
		chart_list = rres.chart_list;
		for (var x = 0; x < chart_list.length; ++x) {
			chart_list[x][5] = new Option(chart_list[x][1], chart_list[x][0]);
			chart_list[x][6] = new Option(chart_list[x][1], chart_list[x][0]);
		}
		accounts_dirty = true;
	}

	if (accounts_dirty) {
		ajax_update_db_accounts();
		ajax_update_cr_accounts();
		accounts_dirty = false;
	}

	if (edit_gotdata_saved) {
		ajax_edit_gotdata(edit_gotdata_saved);
		edit_gotdata_saved = null;
	}
}

function ajax_dbfilter(e) {
	ajax_update_db_accounts();
}

function ajax_crfilter(e) {
	ajax_update_cr_accounts();
}

function auto_stdjournal()
{
	/* Here we try to find a standard journal entry based on the account(s) supplied 
	   in a new journal entry. The heuristics are explained along the code. 

	 TODO: in a future version, we should not try to guess the std. journal entry
	       if the user typed or searched by itself for a code.

	In the future, budget and recurrent journal entries will supply the 
	default standard journals for common journals, so this function may
	be rendered obsolete (a pity since it was fun to write).

	Another possibility would be to query the SQL database in real time,
	deducing the stdjournal frequency for that template.

	*/

	var id = parseInt(document.f.id.value);
	if (id > 0) {
		// the journal is not new, so we don't touch it
		return;
	}

	var db = parseInt(document.f.accountDB.value);
	var cr = parseInt(document.f.accountCR.value);

	if (db <= 0 && cr <= 0) {
		// no account chosen, sell it short
		return;
	}

	// find the standard journals for the account(s) chosen by the user
	var stdjournal = parseInt(document.f.stdjournal.value);
	var db_stdjournal = db > 0 ? -1 : 0;
	var cr_stdjournal = cr > 0 ? -1 : 0;
	var db_result = 0;
	var cr_result = 0;

	// In general, the standard journal is more correlated to the debit accounta
	// Being both accounts equaly good, we have a small bias towards DB-supplied stdjournal
	var heuristic = db > 0 ? -1 : 0;

	for(var x = 0; x < chart_list.length && (db_stdjournal == -1 || cr_stdjournal == -1); ++x) {
		if (db && chart_list[x][0] == db) {
			db_stdjournal = chart_list[x][3];
			db_result = chart_list[x][4];
			// if the account has a stdjournal at all, it earns 100 points
			// if the account is income/expense, 110 points
			// In general, result accounts have more correlation with stdjournals
			// (unless somebody proves me wrong :)
			heuristic -= db_stdjournal > 1 ? 100 + (db_result ? 10 : 0) : 0;
		}
		if (cr && chart_list[x][0] == cr) {
			cr_stdjournal = chart_list[x][3];
			cr_result = chart_list[x][4];
			heuristic += cr_stdjournal > 1 ? 100 + (cr_result ? 10 : 0) : 0;
		}
	}

	if (heuristic != 0) {
		// attribute the standard journal accordingly to heuristic
		v = heuristic > 0 ? cr_stdjournal : db_stdjournal;
		if (v > 1) {
			document.f.stdjournal.value = v;
		}
	}
}

function ajax_dbchange(e) {
	auto_stdjournal();
}

function ajax_crchange(e) {
	auto_stdjournal();
}

function filter_list(db, list, filter, defa2, defa1) {
	filtered_list = [];
	filter = strip(filter);
	for(var x = 0; x < list.length; ++x) {
		// not inactive or inactives not filtered
		var eligible = (! list[x][2]) || (! filter_inactive_accounts);
		if (eligible && filter.length > 0) {
			// ANDs with string filter
			eligible = list[x][1].toLocaleLowerCase().indexOf(filter) > -1;
		}
		if (list[x][0] == 0 || list[x][0] == defa1 || list[x][0] == defa2) {
			// ORs with defaults
			eligible = true;
		}
		if (eligible) {
			filtered_list.push(list[x][5+db]);
		}
	}
	return filtered_list;
}

function ajax_update_db_accounts() {
	var db_saved = document.f.accountDB.value;
	list = filter_list(1, chart_list, strip(document.f.accountDB_filter.value.toLocaleLowerCase()), parseInt(document.f.accountDB.value), db_saved);
	document.f.accountDB.options.length = list.length;
	for(var x = 0; x < list.length; ++x) {
		document.f.accountDB.options[x] = list[x];
	}
	document.f.accountDB.value = db_saved;
	db_saved = 0;
}

function ajax_update_cr_accounts() {
	var cr_saved = document.f.accountCR.value;
	list = filter_list(0, chart_list, strip(document.f.accountCR_filter.value.toLocaleLowerCase()), parseInt(document.f.accountCR.value), cr_saved);
	document.f.accountCR.options.length = list.length;
	for(var x = 0; x < list.length; ++x) {
		document.f.accountCR.options[x] = list[x];
	}
	document.f.accountCR.value = cr_saved;
	cr_saved = 0;
}

function ajax_chart_error() {
	ajax_chart_pending = false;
	alert("Could not load newest version of accounts list");
}

function update_checksum(total_db, total_cr)
{
	swapDOM("total_db", SPAN({"id": "total_db"}, total_db));
	swapDOM("total_cr", SPAN({"id": "total_cr"}, total_cr));
}

function ajax_checksum_error() {
	ajax_checksum_pending = false;
	update_checksum("(error)", "(error)");
}

function ajax_checksum_gotdata(rres)
{
	ajax_checksum_pending = false;

	if (rres.err != "") {
		update_checksum("(error)", "(error)");
	} else {
		update_checksum(rres.total_db, rres.total_cr);
	}
}

function ajax_checksum_update()
{
        var grandmother = parseInt(document.f[grandmother_field].value);
	var mother = parseInt(document.f[mother_field].value);

	if (grandmother <= 0 || mother <= 0) {
		return;
	}

	var jkeys = ["grandmother", "mother"];
	var jvalues = [grandmother, mother];
	var addr = "checksum?" + queryString(jkeys, jvalues);

	var ajax = loadJSONDoc(addr);
	ajax.addCallbacks(ajax_checksum_gotdata, ajax_checksum_error);

	ajax_checksum_pending = true;
}

function ajax_remove_pos(rres)
{
	ajax_checksum_update();
}

function ajax_commit_pos(rres)
{
	ajax_checksum_update();
}

function ajax_cancelled_gotdata(rres)
{
	var statusmsg = "";
	ajax_pending = false;

	if (rres.err != "") {
		alert("Error: "+rres.err);
		return;
	}

	if (rres.cancelled != 0) {
		statusmsg = cancelled_yes;
	} else {
		statusmsg = cancelled_no;
	}

	handle_version(-1, 1);

	xid = "cancelledF"+parseInt(rres.id);
	if (getElement(xid)) {
		getElement(xid).innerHTML = statusmsg;
	}

	ajax_checksum_update();
}

function ajax_cancelled(grandmother,mother,id)
{
	if (ajax_pending) {
		return;
	}

	var jkeys = ["grandmother","mother","id"];
	var jvalues = [grandmother,mother,id];
	var addr = "cancelled?" + queryString(jkeys, jvalues);

	var ajax = loadJSONDoc(addr);
	ajax.addCallbacks(ajax_cancelled_gotdata, ajax_error);

	ajax_pending = true;
}

function ajax_collapse(id, code)
{
	if (hasElementClass(id, "collapsed")) {
		collapse_children(id, 0);
	} else {
		collapse_children(id, 1);
	}
}

function ajax_uncollapse_all()
{
	var a = getElementsByTagAndClassName("tr", "invisible", "tabledata");
	for(var i = 0, el; el = a[i]; ++i) {
		removeElementClass(el, "invisible");
	}
	a = getElementsByTagAndClassName("tr", "collapsed", "tabledata");
	for(var i = 0, el; el = a[i]; ++i) {
		removeElementClass(el, "collapsed");
	}
}

function collapse_children(el, action)
{
	if (action) {
		addElementClass(el, "collapsed");
	} else {
		removeElementClass(el, "collapsed");
	}

	var my_code = getNodeAttribute(el, "code_class");
	var a = getElementsByTagAndClassName("tr", my_code, "tabledata"); 

	for(var i = 0, el; el = a[i]; ++i) {
		if (action) {
			addElementClass(el, "invisible");
		} else {
			removeElementClass(el, "invisible");
		}
	}
}

