import cherrypy

import turbogears
from turbogears import controllers, expose, validate, redirect
from turbogears import identity, validators

from luca.model import cmos, conn, notfound, denied
from luca import model

from luca.simple_table import simple_table

class stdjournals(simple_table):

# SUBCLASS-SPECIFIC FUNCTIONS AND PARAMETERS ##########################################################
	
    validators = {"id": validators.Int(if_invalid=0), 
        	  "description": validators.UnicodeString(if_invalid=u""),
        	  "smalldesc": validators.UnicodeString(if_invalid=u""),
	          "mobile": validators.Int(if_invalid=0), 
		  "inactive":  validators.Int(if_invalid=0)}
				 
    def setup(self):
	self.emptyrow_read = {"id": 0, "description": "", "smalldesc": "", "mobile": 0, "inactive": 0}
	self.emptyrow_write = self.emptyrow_read
	self.empty_intermediate = self.emptyrow_read

	# data for automated tests
	self.sample_data = [{"mobile": 1, "inactive": 0}, {"mobile": 0}, {"mobile": 0}, {"inactive": 1}, {"inactive": 1}]
	self.invalid_data = [("description", "")]
	self.unique_cols = []
	self.foreign_keys = []
	self.test_order = 4
	self.test_flags = ('cant_update_id_1', 'cant_remove_id_1')
	self.test_bump = [ ('cmos', 'stdjournals_version', 1) ]

    	self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
    	self.msg_notfound = "Standard journal not found, probably it has just been deleted"
	self.show_msg_ajax = True
    	self.msg_added = "Standard journal %d has been added"
    	self.msg_updated = "Standard journal %d has been updated"
    	self.msg_removed = "Standard journal %d has been removed sucessfully"
	self.msg_validation = "Order number and description must not be empty"
    	self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
    	self.msg_integrity = "Integrity error: "
    	self.msg_cannotremove = "Standard journal can not be removed"
    	self.msg_unique = "check if the order number is unique"
    	self.msg_legend_update = "Standard journals: editing standard journal %s"
    	self.msg_legend_add = "Standard journals: new standard journal"
    	self.msg_id1_untouchable = "Row #1 cannot be updated or removed!!!"

    	self.table = model.stdjournals
	self.cmos = cmos
	self.streamlined_add = True

    def test__mobile(self, t, id): #pragma: no cover
	t.cargo = t.tu.call(self.mobile, id)
	if not t.cargo["err"]:
		return True
	return False

    def test__inactive(self, t, id): #pragma: no cover
	t.cargo = t.tu.call(self.inactive, id)
	if not t.cargo["err"]:
		return True
	return False

    def test__row_inactive(self, t, row): #pragma: no cover
	t.logmask = 0
	if not t.permission_explode("valid inactive", self.test__inactive, t.DONE, row["id"]):
		t.fail("Failed inactive "+str(row))
	if t.xxx == 0 or t.xxx == 1:
		# keep inactive
		pass
	else:
		if not t.permission_explode("valid inactive", self.test__inactive, t.DONE, row["id"]):
			t.fail("Failed inactive "+str(row))

    def test__row_mobile(self, t, row): #pragma: no cover
	t.logmask = 0
	if not t.permission_explode("valid mobile", self.test__mobile, t.DONE, row["id"]):
		t.fail("Failed mobile "+str(row))

    def test__rows_loop_extra(self, t, row): #pragma: no cover
	self.test__row_mobile(t, row)
	self.test__row_inactive(t, row)

    def data_legend(self, intermediate):
        return unicode(intermediate["id"])+u" - "+intermediate["description"]

    def row_name(self, d_row):
	name = ""
	if d_row["id"] > 0:
		name = u"%s (id %d)" % (d_row["description"], d_row["id"])
	else: 
		name = d_row["description"]
	return name

    def form_to_intermediate_ext(self, form_data, intermediate):
   	intermediate["description"] = intermediate["description"][:60].strip()
   	intermediate["smalldesc"] = intermediate["smalldesc"][:20].strip()
    
    def intermediate_validate(self, d):
	    err = ""
	    if not d["description"]:
		err = "Description must not be empty"
	    if d["id"] == 1:
		err = self.msg_id1_untouchable
	    return err

    def intermediate_to_form_ext(self, intermediate, form_data):
	    # Adds "selected" for <select> fields
	    form_data["mobile_0"] = form_data["mobile_1"] = None
	    if intermediate["mobile"]:
		    form_data["mobile_1"] = "1"
	    else:
		    form_data["mobile_0"] = "1"
	    form_data["inactive_0"] = form_data["inactive_1"] = None
	    if intermediate["inactive"]:
		    form_data["inactive_1"] = "1"
	    else:
		    form_data["inactive_0"] = "1"

# INHERITED METHODS ########################################################

    def trigger_update_post(self, row, id, new, form_data):
	    model.cmos.bump_version("stdjournals")

    def trigger_remove_post(self, row):
	    model.cmos.bump_version("stdjournals")

    def trigger_remove_pre(self, row):
	t = model.journal.select("stdjournal_id = %d" % row["id"])
	for l in t:
	    return self.msg_integrity + "Standard journal description is in use by journal %d:%d:%d" % (l.org.id, l.batch.id, l.id)
	t = model.accounts.select("stdjournal_id = %d" % row["id"])
	for l in t:
	    return self.msg_integrity + "Standard journal description is in use by account %s" % (l.code)
	if row["id"] == 1:
	   return self.msg_id1_untouchable
	return ""

    def table_version(self, row):
	return model.cmos.get_version("stdjournals")


# EXPOSED METHODS ########################################################

    def index_form_data(self):
        return self.base_index(self.table.q.id > 1, orderBy=self.table.q.description)

    @expose(template="luca.templates.stdjournals")
    def index(self, **dummy):
        return self.index_form_data()

    @expose(template="luca.templates.stdjournals_lite")
    def lite(self, **dummy):
        return self.index_form_data()

    @expose("json")
    @validate(validators=validators)
    def remove(self, id, **garbage):
	    result = self.base_remove(id)
	    return result

    @expose()
    @validate(validators=validators)
    def lite_remove(self, id, **garbage):
	    return self.base_lite_remove(id)

    @expose("json")
    @validate(validators=validators)
    def fetch(self, id, **garbage):
	    return self.base_fetch(id)

    @expose(template="luca.templates.stdjournals_lite_form")
    @validate(validators=validators)
    def lite_edit(self, id, **garbage):
	    return self.base_lite_edit(id)

    @expose("json")
    @validate(validators=validators)
    def update(self, **form_data):
	    result = self.base_update(form_data)
	    return result

    @expose(template="luca.templates.stdjournals_lite_form")
    @validate(validators=validators)
    def lite_commit(self,  **form_data):
	    return self.base_lite_commit(form_data)

    def toogle_mobile(self, id):
	    # simulates a data fetch, receives form data
	    form_data = self.base_fetch(id)
	    if form_data["err"]:
		    return form_data
	    if form_data["mobile"]:
	    	form_data["mobile"] = 0
	    else:
	    	form_data["mobile"] = 1
	    form_data["err"] = self.base_update(form_data)["err"]
	    return form_data

    @expose("json")
    @validate(validators=validators)
    def mobile(self, id, **garbage):
	return self.toogle_mobile(id)

    def toogle_inactive(self, id):
	    # simulates a data fetch, receives form data
	    form_data = self.base_fetch(id)
	    if form_data["err"]:
		    return form_data
	    if form_data["inactive"]:
	    	form_data["inactive"] = 0
	    else:
	    	form_data["inactive"] = 1
	    form_data["err"] = self.base_update(form_data)["err"]
	    return form_data

    @expose("json")
    @validate(validators=validators)
    def inactive(self, id, **garbage):
	return self.toogle_inactive(id)

