import cherrypy

import turbogears
from turbogears import controllers, expose, validate
from turbogears import identity, validators
from sqlobject import sqlbuilder

from luca.model import classes, conn, notfound, denied
import model

from luca.simple_table import simple_table

class sys_classes(simple_table):

# SUBCLASS-SPECIFIC FUNCTIONS AND PARAMETERS ##########################################################
	
    validators = {"id": validators.Int(if_invalid=0), 
    		  "oid": validators.Int(if_invalid=0), 
        	  "description": validators.UnicodeString(if_invalid=u""),
	          "is_income": validators.Int(if_invalid=0),
	          "normal_balance": validators.Int(if_invalid=0)}
				 
    def setup(self):
	self.emptyrow_read = {"id": 0, "oid": 0, "description": "", "is_income": 0, "normal_balance": 0}
	self.emptyrow_write = self.emptyrow_read
	self.empty_intermediate = self.emptyrow_read

	# data for automated tests
	self.sample_data = [{"is_income": 1, "normal_balance": 1, "oid": 70}, {"is_income": 0, "normal_balance": 0, "oid": 80}, \
				{"is_income": 0, "normal_balance": 1, "oid": 90}, \
				{"is_income": 0, "normal_balance": 1, "oid": 100} ]

	self.invalid_data = [("oid", -1), ("description", "")]
	self.unique_cols = ["oid"]
	self.foreign_keys = []
	self.test_order = 7
	self.test_bump = []

    	self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
    	self.msg_notfound = "Class not found, probably it has just been deleted"
    	self.msg_added = "Class has been added"
    	self.msg_updated = "Class has been updated"
    	self.msg_removed = "Class has been removed sucessfully"
	self.msg_validation = "Order number and description must not be empty"
    	self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
    	self.msg_integrity = "Integrity error: "
    	self.msg_cannotremove = "Class can not be removed"
    	self.msg_unique = "Make sure that order number is unique"
    	self.msg_legend_update = "Account classes: editing class '%s'"
    	self.msg_legend_add = "Account classes: new class"

    	self.table = classes
	self.streamlined_add = False

	self.formatters = {"is_income": self.format_is_income, "normal_balance": self.format_normal_balance}

    def format_is_income(self, data):
	return data and 'Income (Revenue/Expense)' or 'Assets/Liabilities/Equity'

    def format_normal_balance(self, data):
	return data and 'Debit' or 'Credit'

    def row_name(self, d_row):
	name = ""
	if d_row["id"] > 0:
		name = u"%s (id %d)" % (d_row["description"], d_row["id"])
	else:
		name = d_row["description"]
	return name

    def form_to_intermediate_ext(self, form_data, d):
   	d["description"] = d["description"][:20].strip()
    
    def intermediate_validate(self, d):
	    err = ""
	    if not d["description"]:
		err = "Description must not be empty"
	    elif d["oid"] <= 0:
	        err = "Order number must be positive"
	    if d["id"] > 0:
		if self.table.select(sqlbuilder.AND(self.table.q.oid == d["oid"], self.table.q.id != d["id"])).count() > 0:
			return self.msg_integrity + self.msg_unique
	    else:
		if self.table.select(self.table.q.oid == d["oid"]).count() > 0:
			return self.msg_integrity + self.msg_unique
	    return err

    def data_legend(self, d):
	    return d["description"]

    def intermediate_to_form_ext(self, intermediate, form_data):
	    # Adds "selected" for <select> fields
	    form_data["is_income_0"] = form_data["is_income_1"] = None
	    form_data["normal_balance_0"] = form_data["normal_balance_1"] = None

	    if intermediate["is_income"]:
		    form_data["is_income_1"] = "1"
	    else:
		    form_data["is_income_0"] = "1"
	    if intermediate["normal_balance"]:
		    form_data["normal_balance_1"] = "1"
	    else:
		    form_data["normal_balance_0"] = "1"

    def trigger_remove_pre(self, row):
	t = model.accounts.select("xclass_id = %d" % row["id"])
	for l in t:
	    return self.msg_integrity + "Class is in use by account %d:%s" % (l.chart.id, l.code)

# EXPOSED METHODS ########################################################

    @expose(template="luca.templates.sys_classes")
    def index(self):
        return self.base_index(orderBy=self.table.q.oid)

    @expose(template="luca.templates.sys_classes_lite")
    def lite(self, **garbage):
        return self.base_index(orderBy=self.table.q.oid)

    @expose("json")
    @validate(validators=validators)
    def remove(self, id, **garbage):
	    return self.base_remove(id)

    @expose()
    @validate(validators=validators)
    def lite_remove(self, id, **garbage):
	    return self.base_lite_remove(id)

    @expose("json")
    @validate(validators=validators)
    def fetch(self, id, **garbage):
	    return self.base_fetch(id)

    @expose(template="luca.templates.sys_classes_lite_form")
    @validate(validators=validators)
    def lite_edit(self, id, **garbage):
	    return self.base_lite_edit(id)

    @expose("json")
    @validate(validators=validators)
    def update(self, **form_data):
	    return self.base_update(form_data)

    @expose(template="luca.templates.sys_classes_lite_form")
    @validate(validators=validators)
    def lite_commit(self,  **form_data):
	    return self.base_lite_commit(form_data)

