import cherrypy

import turbogears
from turbogears import controllers, expose, validate, redirect, flash
from turbogears import identity, validators

from luca.model import cmos, conn, notfound, denied

from luca.simple_table import simple_table

class sys_cmos(simple_table):

# SUBCLASS-SPECIFIC FUNCTIONS AND PARAMETERS ##########################################################
	
    validators = {"id": validators.Int(if_invalid=0), 
    		  "batch_zero": validators.Int(if_invalid=0), 
        	  "batch_open": validators.Int(if_invalid=0),
		  "accounts_checkdigit": validators.Int(if_invalid=0)}
				 
    def setup(self):
	self.emptyrow_read = {"id": 0, "batch_zero": 1, "batch_open": 1, "accounts_checkdigit": 1}
	self.emptyrow_write = self.emptyrow_read
	self.empty_intermediate = self.emptyrow_read

	# data for automated tests
	self.sample_data = [{"batch_zero": 1, "batch_open": 1, "accounts_checkdigit": 1}, \
			    {"batch_zero": 0, "batch_open": 0, "accounts_checkdigit": 0}]
	self.invalid_data = [ ("batch_zero", -1), ("batch_zero", 2), \
			       ("batch_open", -1), ("batch_open", 2), \
			       ("accounts_checkdigit", -1), ("accounts_checkdigit", 2) ]
	self.unique_cols = []
	self.foreign_keys = []
	self.test_order = 1000
	self.test_flags = ('cant_insert', 'index_is_fetch', 'cant_fetch_0', 'cant_update_by_id', 'cant_fetch_by_id')
	self.test_bump = []

    	self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
	self.msg_notfound = "ERROR: system parameters not found!"
	self.show_msg_ajax = False
    	# self.msg_added = None # forbidden
    	self.msg_updated = "System parameters have been updated"
    	# self.msg_removed = None # forbidden
	self.msg_validation = "Select valid options!"
    	self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
    	# self.msg_integrity = None # forbidden
    	# self.msg_cannotremove = None # forbidden
    	# self.msg_unique = None # forbidden
    	self.msg_legend_update = "System parameters%s"
    	self.msg_legend_add = self.msg_legend_update

    	self.table = cmos

    def row_name(self, d_row):
	return ""

    def intermediate_validate(self, row):
	if row['batch_open'] < 0 or row['batch_open'] > 1:
		return "Invalid batch_open"
	if row['batch_zero'] < 0 or row['batch_zero'] > 1:
		return "Invalid batch_zero"
	if row['accounts_checkdigit'] < 0 or row['accounts_checkdigit'] > 1:
		return "Invalid accounts_checkdigit"
	return ""

    def data_legend(self, d):
	    return ""

    def intermediate_to_form_ext(self, intermediate, form_data):
	    # Adds "selected" for <select> fields
	    form_data["batch_zero_0"] = form_data["batch_zero_1"] = None
	    form_data["batch_open_0"] = form_data["batch_open_1"] = None
	    form_data["accounts_checkdigit_0"] = form_data["accounts_checkdigit_1"] = None

	    if intermediate["batch_zero"]:
		    form_data["batch_zero_1"] = "1"
	    else:
		    form_data["batch_zero_0"] = "1"

	    if intermediate["batch_open"]:
		    form_data["batch_open_1"] = "1"
	    else:
		    form_data["batch_open_0"] = "1"

	    if intermediate["accounts_checkdigit"]:
		    form_data["accounts_checkdigit_1"] = "1"
	    else:
		    form_data["accounts_checkdigit_0"] = "1"

# OVERRIDEN METHODS #####################################################

    def trigger_remove_pre(self, row): #pragma: no cover
	    # paranoid check
	    return "Can not remove system parameters!"

    def base_fetch(self, id):
	    form_data = super(sys_cmos, self).base_fetch(1)
	    return form_data

    def base_update(self, row):
	row["id"] = 1
	return super(sys_cmos, self).base_update(row)

# EXPOSED METHODS ########################################################

    @expose(template="luca.templates.sys_cmos")
    def index(self):
	    form_data = self.base_fetch(1)
	    # copying behaviour that is only present at simple_table::base_index
	    if self._indexflash:
                self.flash(form_data, self._indexflash, self._indexflash_err)
                self._indexflash = None
	    return form_data

    @expose(template="luca.templates.sys_cmos")
    def lite(self):
	    form_data = self.base_lite_edit(1)
	    # copying behaviour that is only present at simple_table::base_index
	    if self._indexflash:
                self.flash(form_data, self._indexflash, self._indexflash_err)
                self._indexflash = None
	    return form_data

    @expose(template="luca.templates.sys_cmos")
    @validate(validators=validators)
    def lite_commit(self,  **form_data):
	    return self.base_lite_commit(form_data)

    @expose("json")
    @validate(validators=validators)
    def update(self, **form_data):
	    return self.base_update(form_data)

