import cherrypy

import turbogears
from turbogears import controllers, expose, validate, redirect
from turbogears import identity, validators

from luca.model import xuser, conn, notfound, sqlerror, denied
from luca import model
from model import permission

from luca.simple_table import simple_table

class sys_password(simple_table):

# SUBCLASS-SPECIFIC FUNCTIONS AND PARAMETERS ##########################################################
	
    validators = {
	          "old_password": validators.UnicodeString(if_invalid=u""),
	          "new_password": validators.UnicodeString(if_invalid=u""),
	          "new_password2": validators.UnicodeString(if_invalid=u""),
		}
				 
    def setup(self):
	self.vips = ["EVERYBODY"]

    	self.emptyrow_read = {"user_name": "", "email_address": "", "password": "", "display_name": ""}
	self.emptyrow_write = self.emptyrow_read
	self.empty_intermediate = self.emptyrow_read

	# data for automated tests
	self.sample_data = [{"old_password": "admin", "new_password": "admin", "new_password2": "admin"}]
	self.invalid_data = [("old_password", ""), ("new_password", ""), ("new_password2", ""), ("new_password", "xx"), ("new_password2", "yy")]
	self.unique_cols = []
	self.foreign_keys = []

	self.test_order = 1
	self.test_flags = ('noorg_index_allowed', 'anyone_index_allowed', 'cant_fetch_0', 'cant_fetch_by_id', 'cant_update_by_id', 'cant_insert')
	self.test_bump = []

    	self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
    	self.msg_notfound = "User not found, probably it has just been deleted"
	self.show_msg_ajax = False
    	self.msg_updated = "User has been updated"
    	self.msg_validation = ""
    	self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
    	self.msg_integrity = "Integrity error: "
    	self.msg_cannotremove = "User can not be removed"
    	self.msg_cannotremove_admin = "The Administrator cannot be removed!"
    	self.msg_unique = "check if both user name and e-mail are unique"
    	self.msg_legend_update = "System users: editing user '%s'"
    	self.msg_legend_add = "System users: adding new user"

    	self.table = None
	self.streamlined_add = False

    def test__new_row_fixup(self, row): #pragma: no cover
	row["id"] = 1
	row["old_password"] = ""
	row["new_password"] = ""
	row["new_password2"] = ""

    def gen_index(self):
	return {"id": identity.current.user.id, \
		"user_name": identity.current.user.user_name,  \
 	     "display_name":  identity.current.user.display_name, \
	     "email_address":  identity.current.user.email_address, \
	     "err": ""}

    def handle_commit(self, old_password, new_password, new_password2):
	old_password  =  old_password[:40].strip()
	new_password  =  new_password[:40].strip()
	new_password2 = new_password2[:40].strip()

	if identity.current.user.password != old_password:
		return u"Current password incorrect"
	if not new_password:
		return u"Please type a non-blank new password"
	if not new_password2:
		return u"Please type a non-blank new password"
	if new_password != new_password2:
		return u"The two versions of the new password must match"
	try:
		grant = permission(self, "write.edit", 0, 0, "User %d %s" % (identity.current.user.id, identity.current.user.user_name))
		identity.current.user.set(password=new_password)
		grant.done()
		# model.cmos.bump_version("xuser")
	except denied, e: #pragma: no cover
		# should never happen unless SQL error or something as weird
		pass
	except sqlerror, e:  #pragma: no cover
		return self.sqlerr(e)
	return ""

# EXPOSED METHODS ########################################################

    @identity.require(identity.not_anonymous())
    @expose(template="luca.templates.sys_password")
    def index(self):
	form_data = self.gen_index()
	return form_data

    @identity.require(identity.not_anonymous())
    @expose(template="luca.templates.sys_password")
    @validate(validators=validators)
    def update(self, old_password, new_password, new_password2, **garbage):
	err = self.handle_commit(old_password, new_password, new_password2)
	if err: 
		form_data = self.gen_index()
		form_data["err"] = err
		return form_data
	if model._testing_:
		return {"err": ""}

	raise redirect("/") #pragma: no cover

