import cherrypy

import turbogears
from turbogears import controllers, expose, validate
from turbogears import identity, validators
from sqlobject import sqlbuilder

from luca.model import xuser, conn, notfound, denied
from luca import model

from luca.simple_table import simple_table
import time
from mylib import date_formats, thousand_separators, decimal_separators

class sys_users(simple_table):

# SUBCLASS-SPECIFIC FUNCTIONS AND PARAMETERS ##########################################################
	
    validators = {"id": validators.Int(if_invalid=0), 
        	  "user_name": validators.UnicodeString(if_invalid=u""),
	          "email_address": validators.UnicodeString(if_invalid=u""),
	          "password": validators.UnicodeString(if_invalid=u""),
		  "display_name": validators.UnicodeString(if_invalid=u""),
		  "thousand_separator": validators.Int(if_invalid=0),
		  "decimal_separator": validators.Int(if_invalid=0),
		  "date_format": validators.Int(if_invalid=0),
		  "lite":  validators.Int(if_invalid=0) \
		}
				 
    def setup(self):
	# special ACL that is accepted as valid permission
	self.vips = ["ADMIN"]

    	self.emptyrow_read = {"id": 0, "user_name": "", "email_address": "", "password": "", \
			      "display_name": "", "locale": u".,Y/m/d", "lite": 0}
	self.emptyrow_write = self.emptyrow_read
	self.empty_intermediate = self.emptyrow_read

	# data for automated tests
	self.sample_data = [{},{},{},{}]
	self.invalid_data = [("user_name", ""), ("display_name", ""), ("email_address", "")]
	self.invalid_compound_new = [ dict(password="") ]
	self.unique_cols = ["user_name", "email_address"]
	self.foreign_keys = []
	self.test_order = 9
	self.test_flags = ('cant_remove_id_1',)
	self.test_bump = [ ('cmos', 'xuser_version', 1) ]

    	self.msg_unexpected = "ERROR: unexpected SQL error. Contact the system administrator"
    	self.msg_notfound = "User not found, probably it has just been deleted"
	self.show_msg_ajax = False
    	self.msg_added = "User %d has been added"
    	self.msg_updated = "User has been updated"
    	self.msg_removed = "User has been removed sucessfully"
    	self.msg_validation = "Name, nickname and e-mail address must not be empty"
    	self.msg_validation_crash = "Unexpected validation error: notify system administrator!"
    	self.msg_integrity = "Integrity error: "
    	self.msg_cannotremove = "User can not be removed"
    	self.msg_cannotremove_admin = "The Administrator cannot be removed!"
    	self.msg_unique = "Make sure that both user name and e-mail are unique"
    	self.msg_legend_update = "System users: editing user '%s'"
    	self.msg_legend_add = "System users: adding new user"

    	self.table = xuser
	self.streamlined_add = True

	self.thousand_separators = []
	self.thousand_separators_pro = {}
	self.thousand_separators_rev = {}
	x = 0
	for c in thousand_separators:
		self.thousand_separators.append({"id": x, "description": c})
		self.thousand_separators_pro[c] = x
		self.thousand_separators_rev[x] = c
		x += 1

	self.decimal_separators = []
	self.decimal_separators_pro = {}
	self.decimal_separators_rev = {}
	x = 0
	for c in decimal_separators:
		self.decimal_separators.append({"id": x, "description": c})
		self.decimal_separators_pro[c] = x
		self.decimal_separators_rev[x] = c
		x += 1

	self.date_formats = []
	self.date_formats_pro = {}
	self.date_formats_rev = {}
	x = 0
	for f in date_formats.keys():
		self.date_formats.append({"id": x, "description": f})
		self.date_formats_pro[f] = x
		self.date_formats_rev[x] = f
		x += 1

	self.lite_options = [{"id": 0, "description": u"AJAX menu by default"}, {"id": 1, "description": u"'Lite' (no Javascript) menu by default"}]


    def test__verify_excuse(self, col, orig, fetched, orig_row, fetched_row): #pragma: no cover
	return col == "locale" or (col == "password" and fetched == u"")

    def row_name(self, d_row):
	name = ""
	if d_row["id"] > 0:
		name = u"%s (id %d)" % (d_row["user_name"], d_row["id"])
	else:
		name = d_row["user_name"]
	return name

    def intermediate_validate(self, d):
	    err = ""
	    if not d["user_name"]:
		err = "Nickname must not be empty"
	    elif not d["email_address"]:
		err = "E-mail address must not be empty"
	    elif not d["display_name"]:
		err = "User name must not be empty"
	    elif d["id"] <= 0 and not d["password"]:
		err = "New user must have a password"
	    elif len(d["email_address"].encode("utf-8")) > 240: #pragma: no cover
		# paranoid - MySQL's varchar(240) will swallow up to 720 bytes
		err = "E-mail is too long"
	    if d["id"] > 0:
		if self.table.select(sqlbuilder.AND(self.table.q.user_name == d["user_name"], self.table.q.id != d["id"])).count() > 0:
			err = self.msg_integrity + self.msg_unique
		if self.table.select(sqlbuilder.AND(self.table.q.email_address == d["email_address"], self.table.q.id != d["id"])).count() > 0:
			err = self.msg_integrity + self.msg_unique
	    else:
		if self.table.select(self.table.q.user_name == d["user_name"]).count() > 0:
			err = self.msg_integrity + self.msg_unique
		if self.table.select(self.table.q.email_address == d["email_address"]).count() > 0:
			err = self.msg_integrity + self.msg_unique
	    return err

    # def intermediate_to_row_ext(self, intermediate, row):
    #    if not row["password"] and row["id"] > 0:
    #        del row["password"]

    def data_legend(self, d):
	    return d["user_name"]

    def intermediate_to_form_ext(self, intermediate, form_data):
	form_data["password"] = ""
	if form_data["id"] <= 0:
		# copies default locale settings from administrator, so configure the admin
		# as the defaults you like!
		form_data["locale"] = self.table.get(1).locale
	if len(form_data["locale"]) < 7:
		# paranoid check (since data comes from database)
		form_data["locale"] = ".,Y/m/d" #pragma: no cover
	try:
		form_data["thousand_separator"] = self.thousand_separators_pro[form_data["locale"][0]]
		form_data["decimal_separator"] = self.decimal_separators_pro[form_data["locale"][1]]
		form_data["date_format"] = self.date_formats_pro[form_data["locale"][2:7]]
	except KeyError: #pragma: no cover
		# paranoid, should never happen since this data comes from database
		form_data["thousand_separator"] = 0
		form_data["decimal_separator"] = 0
		form_data["date_format"] = 0
		
    def form_to_intermediate_ext(self, form_data, intermediate):
   	intermediate["user_name"] = intermediate["user_name"][:16].strip()
	intermediate["email_address"] = intermediate["email_address"][:100].strip()
	intermediate["password"] = intermediate["password"][:40].strip()
	intermediate["display_name"] = intermediate["display_name"][:40].strip()
	try:
		intermediate["locale"] = self.thousand_separators_rev[form_data["thousand_separator"]] + self.decimal_separators_rev[form_data["decimal_separator"]] + self.date_formats_rev[form_data["date_format"]]
	except KeyError:
		intermediate["locale"] = u".,Y/m/d"

    def auxiliary_form_data(self, form_data):
	form_data["date_formats"] = self.date_formats
	form_data["thousand_separators"] = self.thousand_separators
	form_data["decimal_separators"] = self.decimal_separators
	form_data["lite_options"] = self.lite_options
    
    def trigger_remove_pre(self, row):
	if row["id"] <= 1:
		return self.msg_cannotremove_admin
	t = model.log.select("user_id = %d" % row["id"])
	for l in t:
	    return self.msg_integrity + "User has been logged"
	return ""

    def trigger_update_post(self, row, id, new, form_data):
	    model.cmos.bump_version("xuser")

    def trigger_remove_post(self, row):
	    model.cmos.bump_version("xuser")

    def table_version(self, row):
	return model.cmos.get_version("xuser")

# EXPOSED METHODS ########################################################


    @expose(template="luca.templates.sys_users")
    def index(self):
        return self.base_index(orderBy=self.table.q.user_name)

    @expose(template="luca.templates.sys_users_lite")
    def lite(self, **garbage):
        return self.base_index(orderBy=self.table.q.user_name)

    @expose("json")
    @validate(validators=validators)
    def remove(self, id, **garbage):
	    return self.base_remove(id)

    @expose()
    @validate(validators=validators)
    def lite_remove(self, id, **garbage):
	    return self.base_lite_remove(id)

    @expose("json")
    @validate(validators=validators)
    def fetch(self, id, **garbage):
	    return self.base_fetch(id)

    @expose(template="luca.templates.sys_users_lite_form")
    @validate(validators=validators)
    def lite_edit(self, id, **garbage):
	    return self.base_lite_edit(id)

    @expose("json")
    @validate(validators=validators)
    def update(self, **form_data):
	    return self.base_update(form_data)

    @expose(template="luca.templates.sys_users_lite_form")
    @validate(validators=validators)
    def lite_commit(self,  **form_data):
	    return self.base_lite_commit(form_data)

