# -*- encoding: utf-8 -*-

import cherrypy
from turbogears import identity
from luca.controllers import Root
from luca import model, mylib
import turbogears
import random
import types
import sys
import os
from luca.simple_report import simple_report
from luca.simple_table import simple_table

DONE = model.permission.MASK_DONE
FAIL = model.permission.MASK_FAIL
DENIED = model.permission.MASK_DENIED

class test_engine:
	''' Test engine: deals with permissions and makes the cartesian explosion of calls 
	    w/o permissions and with permissions. '''

	def __init__(self):
		# variables that drive permission_explode()
		self.r00t = None
		self.tu = None

		self.test_name = []
		self.logmask = 0
		self.C = None
		self.cargo = None   # intermediate for detailed checks
		self.forgive_org_permission_hole = 0
		self.forgive_permission_hole = 0
		self.my_lastlog = 0
		self.DONE = DONE
		self.FAIL = FAIL
		self.DENIED = DENIED

	def tag(self):
		model._testing_tag_ += 1
		return model._testing_tag_

	def fail(self, err):
		print self.cargo
		print self.test_name
		print err
		turbogears.startup.stopTurboGears()
		sys.exit(1)

	def permission_login(self, user_id):
		u = model.xuser.get(user_id)
		self.tu.set_identity_user(u)   # needs Turbogears to be patched (ticket 1166) or be version >= 1.0.1

	def permission_setup(self):
		self.permission_login(1)
		ret = self.tu.call(self.r00t.sys_users.update, id=-1, user_name=u"winner", email_address=u"winner", password=u"admin", display_name="winner", locale="", lite=0)
		if ret['err']: self.fail(ret['err'])
		ret = self.tu.call(self.r00t.sys_users.update, id=-1, user_name=u"loser", email_address=u"loser", password=u"admin", display_name="loser", locale="", lite=0)
		if ret['err']: self.fail(ret['err'])
		if model.xuser.get(3).user_name != u"loser":
			self.fail("User ID 3 is not loser")
	
		ret = self.tu.call(self.r00t.sys_groups.update, id=-1, group_name=u"winner", display_name="winner")
		if ret['err']: self.fail(ret['err'])
		ret = self.tu.call(self.r00t.sys_groups.update, id=-1, group_name=u"loser", display_name="loser")
		if ret['err']: self.fail(ret['err'])
		if model.xgroup.get(3).group_name != u"loser":
			self.fail("Group ID 3 is not loser")
	
		ret = self.tu.call(self.r00t.sys_usersXgroups.index, select_parent=1, parent_now_id=1, button_del="x", employed=[1])
		for p in model.xuser.get(1).groups:
			if p.id == 1:
				break
		else:
			self.fail("Should have failed removal of user 1 from group 1")
	
		ret = self.tu.call(self.r00t.sys_groupsXperms.lite, select_parent=1, parent_now_id=1, button_del="x", employed=[1])
		for p in model.xgroup.get(1).permissions:
			if p.id == 1:
				break
		else:
			self.fail("Should have failed removal of group 1 from permission 1 (g0d)")
	
		ret = self.tu.call(self.r00t.sys_usersXgroups.lite, select_parent=2, parent_now_id=2, button_add="x", available=[2])
		ret = self.tu.call(self.r00t.sys_usersXgroups.index, select_parent=3, parent_now_id=3, button_add="x", available=[3])
	
		winner = model.xgroup.get(2)
		for m in model.modules:
			for p in model.function_points:
				n = model.xpermission.by_permission_name(m+"."+p)
				winner.addXpermission(n)
	
		if model.xuser.get(3).groups[0].id != model.xuser.by_user_name("loser").id:
			self.fail("Group ID 3 is not related to User 3")
	
	def permission_add(self, group_id, perm):
		if perm == "EVERYBODY":
			# placeholder just for testing; ignore
			return
		self.permission_login(1)
		permission_id = model.xpermission.by_permission_name(perm).id
		ret = self.tu.call(self.r00t.sys_groupsXperms.index, select_parent=group_id, parent_now_id=group_id, \
								button_add="x", available=[permission_id])
	
		for p in model.xgroup.get(group_id).permissions:
			if p.permission_name == perm:
				break
		else:
			self.fail("Permission add %d %s failed" % (group_id, perm))

	def permission_del(self, group_id, perm):
		if perm == "EVERYBODY":
			# placeholder just for testing; ignore
			return
		self.permission_login(1)
		permission_id = model.xpermission.by_permission_name(perm).id
		ret = self.tu.call(self.r00t.sys_groupsXperms.index, select_parent=group_id, parent_now_id=group_id, \
								button_del="x", employed=[permission_id])
	
		for perm in model.xgroup.get(group_id).permissions:
			if perm.permission_name == perm:
				self.fail("Permission del %d %s failed" % (group_id, perm))

	def permission_add_orgs(self, user_id):
		self.permission_login(1)
		orgs_list = []
		for org in model.orgs.select():
			orgs_list.append(org.id)	
	
		ret = self.tu.call(self.r00t.sys_usersXorgs.lite, select_parent=user_id, parent_now_id=user_id, \
								button_add="x", available=orgs_list)
		if len(model.xuser.get(user_id).orgs) < len(orgs_list):
			self.fail("Orgs permission add %d failed" % (user_id,))


	def permission_del_orgs(self, user_id):
		self.permission_login(1)
		orgs_list = []
		for org in model.orgs.select():
			orgs_list.append(org.id)	
	
		ret = self.tu.call(self.r00t.sys_usersXorgs.index, select_parent=user_id, parent_now_id=user_id, \
								button_del="x", employed=orgs_list)

		if len(model.xuser.get(user_id).orgs) > 0:
			self.fail("Orgs permission del %d failed" % (user_id,))

	def permission_checklog_pre(self):
		if not model.lastlog:
			self.my_lastlog = 0
		self.my_lastlog = model.lastlog.id

	def permission_checklog(self, expected_log, logmask, *params):
		expected = expected_log & ~logmask
		try:
			# print "Looking for %d" % (lastlog+1)
			has_row = model.log.get(self.my_lastlog + 1)
		except model.notfound:
			has_row = None
		if (has_row and not expected):
			self.fail("Unexpected log last_id=%d select_id=%d detected=%x mask=%x\n\nMost common cause: transaction should have failed in validation but failed in permission\n\nmsg=%s %s\n\n%s" % (self.my_lastlog, self.my_lastlog+1, expected_log, logmask, has_row.description, has_row.err, params))
		if (not has_row and expected):
			self.fail("Log row not found! last_id=%d select_id=%d expected=%x mask=%x\n\n%s" % (self.my_lastlog, self.my_lastlog+1, expected_log, logmask, params))

	def permission_explode(self, name, test_function, expected_log, *params):
		''' Tries the test_function operation several times, all should fail,
		    except the last one that must succeed.
		'''
	
		self.test_name.append(name)
		self.test_name.append("")
	
		winner, loser = (2, 3)
		if 'admin_as_winner' in self.flags():
			# AVOIDS deadlock (integrity error when removing org when user refers to org)
			winner = 1
	
		for perm in self.acl_vips():
			self.permission_add(winner, perm)

		self.permission_login(loser)
		self.test_name[-1] = "perm:loser"
		print unicode(self.tag()), " ".join(self.test_name), test_function.__name__
	
		self.permission_checklog_pre()
		if test_function(self, *params) and not self.forgive_permission_hole:
			self.fail("Test should have failed")
		if not self.forgive_permission_hole:
			self.permission_checklog(DENIED, self.logmask, params)
	
		self.permission_login(winner)
		org_permission = -1
		if self.org_perm():
			self.test_name[-1] = "perm:no_org"
			print unicode(self.tag()), " ".join(self.test_name), test_function.__name__
			self.permission_checklog_pre()
			if test_function(self, *params) and not self.forgive_org_permission_hole:
				self.fail("Test should have failed")
			if not self.forgive_org_permission_hole:
				self.permission_checklog(DENIED, self.logmask, params)
			org_permission = random.randint(0,1)
			if winner == 1:
				# AVOIDS deadlock (integrity error when removing org when user refers to org)
				org_permission = 0
			if org_permission == 1:
				self.permission_add_orgs(winner) # logs as admin 
			else:
				self.permission_add(winner, "ALL_ORGS")
			self.permission_login(winner)
	
		self.test_name[-1] = "perm:winner"
		self.permission_checklog_pre()
	
		print unicode(self.tag()), " ".join(self.test_name), test_function.__name__
	
		result = test_function(self, *params)
	
		self.permission_checklog(result and DONE or FAIL, self.logmask, params)
	
		# cleanup
		for perm in self.acl_vips():
			self.permission_del(winner, perm)
		if org_permission == 1:
			self.permission_del_orgs(winner)
		elif org_permission == 0:
			self.permission_del(winner, "ALL_ORGS")
	
		del self.test_name[-2:]
	
		return result
	
	def permission_granted(self, name, test_function, expected_log, *params):
		self.test_name.append(name)
		self.test_name.append("")
	
		winner, loser = (2, 3)
		if 'deadlock_on_remove' in self.flags():
		# AVOIDS deadlock (integrity error when removing org when user refers to org)
			winner = 1
	
		for perm in self.acl_vips():
			self.permission_add(winner, perm)
	
		self.permission_login(winner)
		org_permission = -1
		if self.org_perm():
			org_permission = random.randint(0,1)
			if winner == 1:
				# AVOIDS deadlock (integrity error when removing org when user refers to org)
				org_permission = 0
			if org_permission == 1:
				self.permission_add_orgs(winner) # logs as admin 
			else:
				self.permission_add(winner, "ALL_ORGS")
			self.permission_login(winner)
	
		self.test_name[-1] = "perm:GRANTED"
		self.permission_checklog_pre()
	
		print unicode(self.tag()), " ".join(self.test_name), test_function.__name__
	
		result = test_function(self, *params)
	
		self.permission_checklog(result and DONE or FAIL, self.logmask, params)
	
		# cleanup
		for perm in self.acl_vips():
			self.permission_del(winner, perm)
		if org_permission == 1:
			self.permission_del_orgs(winner)
		elif org_permission == 0:
			self.permission_del(winner, "ALL_ORGS")
	
		del self.test_name[-2:]
	
		return result

	def flags(self):
		return self.C.test_flags

	def acl_vips(self):
		return self.C.vips
	
	def org_perm(self):	
		return self.C.perm_org({"id": 1, "org": 1}) != 0


##########################################################


def test_main(r00t):
	t = test_engine()

	t.r00t = r00t
	import cherrypy
	cherrypy.root = r00t
	from turbogears import testutil
	t.tu = testutil
	t.permission_setup()

	t.test_name.append("r00t")
	r00t.test__run(t)
	del t.test_name[-1:]

	# grabs all controllers that are subclasses of simple_table
	tables = [x for x in dir(r00t) if isinstance(getattr(r00t, x), simple_table)]
	# sort the table based on controller.test_order, because the test data is order-dependent
	tables.sort(lambda x,y: cmp(getattr(r00t, x).test_order, getattr(r00t, y).test_order))

	for test in tables:
		if test == 'currencies' or test == 'currency_rates':
			continue
		t.forgive_org_permission_hole = 0
		t.forgive_permission_hole = 0
		t.logmask = 0
		t.test_name.append(test)
		getattr(r00t, test).test__run(t)
		del t.test_name[-1:]

	model.cmos.get(1).set(batch_open=1, batch_zero=1)

	for test in [x for x in dir(r00t) if isinstance(getattr(r00t, x), simple_report)]:
		t.forgive_org_permission_hole = 0
		t.forgive_permission_hole = 0
		t.logmask = 0
		t.test_name.append(test)
		getattr(r00t, test).test__run(t)
		del t.test_name[-1:]

	t.test_name.append("BALANCE CLOSING")

	t.forgive_org_permission_hole = 0
	t.forgive_permission_hole = 0
	t.logmask = 0
	r00t.balance_closing.test__run(t)

	del t.test_name[-1:]

	print
	print
	print "OK OK OK OK OK OK OK OK OK OK OK OK OK OK"
	print
	print

