#!/usr/bin/env python

# Copyright (c) 2005 INdT - Instituto Nokia de Tecnologia, Elvis Pfutzenreuter.
# All rights reserved. This software is released under the LGPL licence.

import appuifw
import sys

sys.path.append("E:\\Python")

import face_s60
import protocol
import client_database
import bluetooth_s60
import mylib
import anydbm
# import time
import socket
import random

repo_journal = client_database.journal(anydbm, u"E:\\journal", fast=False)
repo_stdjournals = client_database.stdjournals(anydbm, u"E:\\stdjournals", fast=False)
repo_accounts = client_database.accounts(anydbm, u"E:\\accounts", fast=False)
repo_cmos = client_database.cmos(anydbm, u"E:\\cmos", fast=False)
	
protocol.register_net_exceptions(bluetooth_s60.exceptions)

def init():
	repo_journal.truncate()
	repo_stdjournals.truncate()
	repo_accounts.truncate()
	repo_cmos.truncate()

def sync(request, host):
	if request < 1:
		return ("Internal Error", True)
	
	ret = "Done"
	err = False

	try:
		sock = bluetooth_s60.client_socket(host)
		c = protocol.client(sock, repo_cmos, repo_stdjournals, repo_accounts, repo_journal)
		work = 2
		while c.active:
			# time.sleep(1)
			if work <= 0:
				raise Exception("No usable work done in 2 loops!")
			work = work - 1
	
			if c.tick():
				work = 2
				# Client may have deactivated in this point
				continue

			rdneeded, wrneeded  = c.sock_needs()
			rd, wr = protocol.wait_one(rdneeded, wrneeded, nowrite=1)
				
			if rd:
				c.tick_read()
				work = 2
			if wr:
				c.tick_write()
				work = 2

		del sock
		del c

	except (mylib.CommError, mylib.DatamodelError), e:
		ret = u"Error: "+unicode(e)
		err = True

	return (ret, err)

def run():
	f = face_s60.face_s60(repo_journal, repo_accounts, sync, init)
	f.main()
	del f

if __name__ == '__main__':
	run()

