## -*- coding: utf-8 -*-

# Copyright (c) 2005 INdT - Instituto Nokia de Tecnologia, Elvis Pfutzenreuter.
# All rights reserved. This software is released under the LGPL licence.


import gtk
import gobject
is_hildon = 1
try:
	import hildon
except ImportError:
	is_hildon = 0

from mylib import *

BROWSE = 1
STUCK = 2
ADD = 3
ALT = 4
REMOVE = 5
SYNCING = 6

class face_hildon:
	def left(self, widget, data=None):
		self.refresh_journal(-1)
		self.msg(u"")
	
	def right(self, widget, data=None):
		self.refresh_journal(1)
		self.msg(u"")
	
	def first(self, widget, data=None):
		self.refresh_journal(-999999999)
		self.msg(u"")
	
	def last(self, widget, data=None):
		self.refresh_journal(999999999)
		self.msg(u"")
	
	def add(self, widget, data=None):
		sdate = dtoc(today())
		self.fields["date"].set_text(sdate)
		self.fields["value"].set_text(u"")
		self.mode = ADD
		self.msg(u"New journal entry")
		self.refresh_controls()
	
	def _add_commit(self):
		new_journals, err = self.chk_journal(1)
		if not new_journals:
			self.msg(err)
			return

		for new_journal in new_journals:
			try:
				self.journal_position = self.repo_journal.append(new_journal)

			except DatamodelError, e:
				self.modal(u"Error: %s" % unicode(e))
				break
		else:
			self.msg(u"Journal entry added.")

		self.refresh_journal()

	def remove(self, widget, data=None):
		self.mode = REMOVE
		self.msg(u"Confirm journal entry removal")
		self.refresh_controls()

	def _remove_commit(self):
		try:
			self.repo_journal.remove(self.present_journal["key"])
			self.msg(u"Journal entry removed.")

		except DatamodelError, e:
			self.modal(u"Error: %s" % unicode(e))

		self.refresh_journal()
	
	def alt(self, widget, data=None):
		self.mode = ALT
		self.msg(u"Journal entry update")
		self.refresh_controls()

	def _alt_commit(self):
		changed_journal, err = self.chk_journal(0)
		if not changed_journal:
			self.msg(err)
			return

		changed_journal = changed_journal[0]

		changed_journal["key"] = self.present_journal["key"]
		try:
			self.journal_position = self.repo_journal.update(changed_journal)
			self.msg(u"Journal entry changed.")

		except DatamodelError, e:
			self.modal(u"Error: %s" % unicode(e))
		
		self.refresh_journal()

	def yes(self, widget, data=None):
		if self.mode == ADD:
			self._add_commit()
		elif self.mode == ALT:
			self._alt_commit()
		elif self.mode == REMOVE:
			self._remove_commit()

	def no(self, widget, data=None):
		self.refresh_journal()
		self.msg(u"Operation cancelled.")

	def refresh_screen(self):
		while gtk.events_pending():
			gtk.main_iteration(1)
	
	def menu_init(self, widget, data=None):
		if self.mode != STUCK and self.mode != BROWSE:
			return
		self.init_handler()
		self.load_accounts()
		self.fields["db"].set_active(0)
		self.fields["cr"].set_active(0)
		self.refresh_journal()
		self.msg("Database is empty")

	def menu_sync(self, widget, data=None):
		if self.mode != STUCK and self.mode != BROWSE:
			return

		# modal window for the user to choose the server

		self.msg("Searching for Bluetooth devices...")
		self.refresh_screen()
		list, err = self.sync_handler(0, "")
		if err:
			self.msg(err)
			return
		self.msg("")

		self.btwindow = gtk.Window()
		self.btwindow.set_modal(True)
		self.btwindow.set_title("Choose Bluetooth device")
		self.btwindow.set_transient_for(self.window)
		self.btwindow.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
		
		pack = gtk.VBox()
		buttons = gtk.HButtonBox()
		buttons.set_layout(gtk.BUTTONBOX_SPREAD)

		self.hostmodel = gtk.TreeStore(gobject.TYPE_PYOBJECT, gobject.TYPE_STRING)
		self.host = gtk.TreeView(self.hostmodel)
		renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Host", renderer, text=1)
		self.host.append_column(column)

		for h in list:
			iter = self.hostmodel.insert_before(None, None)
			self.hostmodel.set_value(iter, 0, h)
			self.hostmodel.set_value(iter, 1, "%s (%s)" % (h[1], h[0]))
		if list:
			pass

		label = gtk.Label()
		space = gtk.Label()
		bok = gtk.Button(stock=gtk.STOCK_OK)
		bcancel = gtk.Button(stock=gtk.STOCK_CANCEL)

		# gets defaull URL from syncweb handler
		label.set_text("")

		label.show()
		pack.pack_start(label)
		self.host.show()
		pack.pack_start(self.host)
		space.show()
		pack.pack_start(space)

		bok.connect("clicked", self.menu_sync2)
		bcancel.connect("clicked", self.menu_sync_cancel)
		bok.show()
		bcancel.show()
		buttons.pack_start(bok)
		buttons.pack_start(bcancel)

		buttons.show()
		pack.pack_start(buttons)
		pack.show()
		self.btwindow.add(pack)
		self.btwindow.show()

		return

	def menu_sync_cancel(self, widget, data=None):
		self.btwindow.destroy()
		self.btwindow = None

	def menu_sync2(self, widget, data=None):
		host = ''
		model, iter = self.host.get_selection().get_selected()
		if iter:
			h = model.get_value(iter, 0)
			host = h[0]
			print "Will connect to %s" % host

		self.btwindow.destroy()
		self.btwindow = None
		self.refresh_screen()
		
		if self.mode != STUCK and self.mode != BROWSE:
			return

		if not host:
			return

		self.mode = SYNCING
		self.refresh_controls()
		self.msg(u"Syncing via Bluetooth ...")
		self.refresh_screen()

		diff = self.repo_journal.balance()
		if diff:
			self.refresh_journal()
			self.modal(u"Journal is not balanced, difference is %.2f" % (diff / 100.0))
			return

		ret = self.sync_handler(1, host)
		self.modal(ret)
		self.msg("")
		self.mode = BROWSE
		self.load_accounts()
		self.fields["db"].set_active(0)
		self.fields["cr"].set_active(0)
		self.refresh_journal()

	def menu_syncweb(self, widget, data=None):
		if self.mode != STUCK and self.mode != BROWSE:
			return

		# modal window for the user to choose the server

		self.urlwindow = gtk.Window()
		self.urlwindow.set_modal(True)
		self.urlwindow.set_title("Choose Web server")
		self.urlwindow.set_transient_for(self.window)
		self.urlwindow.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
		
		pack = gtk.VBox()
		buttons = gtk.HButtonBox()
		buttons.set_layout(gtk.BUTTONBOX_SPREAD)
		self.url = gtk.Entry()
		label = gtk.Label()
		space = gtk.Label()
		bok = gtk.Button(stock=gtk.STOCK_OK)
		bcancel = gtk.Button(stock=gtk.STOCK_CANCEL)

		# gets defaull URL from syncweb handler
		self.url.set_text(self.syncweb_handler(0, None))
		self.url.set_property("width-chars", 30)
		label.set_text("Web address:")

		label.show()
		pack.pack_start(label)
		self.url.show()
		pack.pack_start(self.url)
		space.show()
		pack.pack_start(space)

		bok.connect("clicked", self.menu_syncweb2)
		bcancel.connect("clicked", self.menu_syncweb_cancel)
		bok.show()
		bcancel.show()
		buttons.pack_start(bok)
		buttons.pack_start(bcancel)

		buttons.show()
		pack.pack_start(buttons)
		pack.show()
		self.urlwindow.add(pack)
		self.urlwindow.show()

		return

	def menu_syncweb_cancel(self, widget, data=None):
		self.urlwindow.destroy()
		self.urlwindow = None

	def menu_syncweb2(self, widget, data=None):
		url = self.url.get_text()

		self.urlwindow.destroy()
		self.urlwindow = None

		self.refresh_screen()
		
		if self.mode != STUCK and self.mode != BROWSE:
			return

		self.mode = SYNCING
		self.refresh_controls()
		self.msg(u"Syncing via Web ...")
		self.refresh_screen()

		diff = self.repo_journal.balance()
		if diff:
			self.refresh_journal()
			self.modal(u"Journal is not balanced, difference is %.2f" % (diff / 100.0))
			return

		ret = self.syncweb_handler(1, url)
		self.modal(ret)
		self.msg("")
		self.mode = BROWSE
		self.load_accounts()
		self.fields["db"].set_active(0)
		self.fields["cr"].set_active(0)
		self.refresh_journal()

	def menu_quit(self, widget, data=None):
		gtk.main_quit()

	def menu_quit(self, widget, data=None):
		gtk.main_quit()

	def refresh_controls(self):
		if self.mode == BROWSE:
			for f in self.buttons.values():
				f.set_sensitive(True)
			self.buttons["yes"].set_sensitive(False)
			self.buttons["no"].set_sensitive(False)
			if self.journal_position == 0:
				self.buttons["left"].set_sensitive(False)
				self.buttons["first"].set_sensitive(False)
			elif ( self.journal_position + 1 ) >= self.repo_journal.count():
				self.buttons["right"].set_sensitive(False)
				self.buttons["last"].set_sensitive(False)

			for f in self.fields.values():
				if not isinstance(f, gtk.ComboBox):
					f.set_editable(False)

		elif self.mode == SYNCING:
			for f in self.buttons.values():
				f.set_sensitive(False)
			for f in self.fields.values():
				if not isinstance(f, gtk.ComboBox):
					f.set_editable(False)

		elif self.mode == STUCK:
			for f in self.buttons.values():
				f.set_sensitive(False)
			self.buttons["add"].set_sensitive(True)
			self.buttons["yes"].set_sensitive(False)
			for f in self.fields.values():
				if not isinstance(f, gtk.ComboBox):
					f.set_editable(False)

		elif self.mode == ADD or self.mode == ALT or self.mode == REMOVE:
			for f in self.buttons.values():
				f.set_sensitive(False)
			self.buttons["yes"].set_sensitive(True)
			self.buttons["no"].set_sensitive(True)
			for f in self.fields.values():
				if not isinstance(f, gtk.ComboBox):
					f.set_editable(True)

		else:
			raise Exception("Unknown mode!")

	def refresh_journal(self, delta = 0):
		self.mode = BROWSE
		self.journal_position += delta

		if self.journal_position < 0:
			self.journal_position = 0
		if self.journal_position >= self.repo_journal.count():
			self.journal_position = self.repo_journal.count() - 1

		if self.journal_position >= 0:
			try:
				self.present_journal = self.repo_journal.get_by_pos(self.journal_position)
			except CommError, e:
				self.modal(repr(e))
			self.show_journal()
		else:
			self.mode = STUCK

			for f in self.fields.values():
				if not isinstance(f, gtk.ComboBox):
					f.set_text(u"")

		self.refresh_controls()

	# General event/window handling section

	def msg(self, s):
		self.field_msg.set_text(s)

	def modal(self, s):
		d = gtk.MessageDialog(parent=None, flags=0, type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_OK, message_format=s)
		d.run()
		d.destroy()

	def delete_event(self, widget, event, data=None):
		return False

	def destroy(self, widget, data=None):
		gtk.main_quit()

	# Data manipulation section

	def chk_date(self, sdate, is_add):
		ndate = ctod(sdate)
	
		if not ndate:
			return u"Invalid date"
		
		if is_add and abs((today() - ndate)) > 28*DAYS:
			return u"Date too far apart from today"
		elif not is_add and abs((today() - ndate)) > 28*DAYS:
			return u"Date too far apart from today"
			
		return None
		
	def chk_value(self, svalue):
		svalue = svalue.replace(u',', u'')
		svalue = svalue.replace(u'.', u'')
		try:
			nvalue = int(svalue)
		except ValueError:
			return(0, svalue, u"Invalid value")
		
		svalue = u"%.2f" % (nvalue / 100.0)

		return (nvalue, svalue, None)

	def chk_journal(self, is_add=0):
		if self.fields["db"].get_active() > 0 and self.fields["cr"].get_active() > 0:
			if not is_add:
				return (None, u"Choose either debit or credit account, not both")

		if self.fields["db"].get_active() <= 0 and self.fields["cr"]. get_active() <= 0:
			return (None, u"Choose either debit or credit account")

		if self.fields["db"].get_active() == self.fields["cr"].get_active():
			return (None, u"Debit and credit accounts must be different")

		nvalue, cvalue, err = self.chk_value(self.fields["value"].get_text().decode())
		self.fields["value"].set_text(cvalue)
		if err:
			return (None, err)

		cdate = self.fields["date"].get_text().decode()
		err = self.chk_date(self.fields["date"].get_text().decode(), is_add)
		if err:
			return (None, err)
		ndate = ctod(cdate)

		descr = self.fields["description"].get_text().decode().strip()[0:60]
		self.fields["description"].set_text(descr)

		if not descr:
			err = u"Description is empty"
			return (None, err)

		ved = []
		tmstamp = now()
		
		if self.fields["db"].get_active() > 0:
			ed = {}
			ed["type"] = u"journalentry"
			ed["description"] = self.fields["description"].get_text().decode()
			ed["account"] = self.accounts_list[self.fields["db"].get_active() - 1][0]
			ed["date"] = ndate
			ed["value"] = -nvalue
			ed["stdjournal"] = u'1' # by now
	                ed["timestamp"] = tmstamp
			ved.append(ed)
		
		if self.fields["cr"].get_active() > 0:
			ed = {}
			ed["type"] = u"journalentry"
			ed["description"] = self.fields["description"].get_text().decode()
			ed["account"] = self.accounts_list[self.fields["cr"].get_active() - 1][0]
			ed["date"] = ndate
			ed["value"] = nvalue
			ed["stdjournal"] = u'1' # by now
	                ed["timestamp"] = tmstamp + 1
			ved.append(ed)

		return (ved, "")
	
	def account_index(self, account_number):
		i = 0
		for a in self.accounts_list:
			# no, there is no off-by-one bug here, in combobox item 0 is the '--'
			i = i + 1
			if a[0] == account_number:
				break
		else:
			i = 0
		return i
		
	def show_journal(self):
		de = self.present_journal
		self.fields["description"].set_text(de["description"])
		if de["value"] < 0:
			self.fields["db"].set_active(self.account_index(de["account"]))
			self.fields["cr"].set_active(0)
		else:
			self.fields["cr"].set_active(self.account_index(de["account"]))
			self.fields["db"].set_active(0)

		self.fields["date"].set_text(dtoc(de["date"]))
		self.fields["value"].set_text(u"%.2f" % (abs(de["value"]) / 100.0))

	def load_accounts(self):
		self._accountlist.clear()
		self._accountlist.append([u'--', u'--'])

		try:
			self.accounts_list = self.repo_accounts.get_list()
		except CommError, e:
			self.modal(repr(e))
			return

		for a in self.accounts_list:
			self._accountlist.append([a[1], a[1]])

	# Window creation section

	def _datafield(self, name, screenfield, size, align, maxlength=0):
		''' Helper function for window creation. Called just from __init__ '''

		l = gtk.Label(name)
		l.set_alignment(1, 0.5) 

		self.labels.append(l)

		if isinstance(screenfield, gtk.Entry):
			e = screenfield
		else:
			e = screenfield.child;

		if isinstance(screenfield, gtk.Entry):
			e.set_alignment(align) # isto é alinhamento do texto *dentro* do campo
						# e não do campo em si.
			e.set_property("width-chars", size)
			if not maxlength:
				maxlength = size
			e.set_max_length(maxlength)

		# Para que o campo não seja centralizado pela tabela, o jeito foi criar um
		# HBox que contém o campo e um Label expansível à direita.
		#
		# Do contrário, Table estica o campo, ignorando a propriedade width-chars
		# setada logo acima.

		hbox = gtk.HBox()
		hbox.pack_start(screenfield, False, False, 0)
		hbox.pack_start(gtk.Label(u""), False, False, 0)
		hbox.show()
		
		datatable.attach(l, 0, 1, len(self.labels)-1, len(self.labels), gtk.FILL, 0)
		datatable.attach(hbox, 1, 2, len(self.labels)-1, len(self.labels), gtk.FILL, 0)

		l.show()
		screenfield.show()

	def _navbutton(self, type, callback):
		''' Helper function for window creation. Called just from __init__ '''
		b = gtk.ToolButton(type)
		self.navbuttonsbox.pack_start(b)
		b.connect("clicked", callback)
		b.show()
		return b

	def _menuitem(self, icon, txt, cbmethod):
		img = gtk.Image()
		img.set_from_stock(icon, gtk.ICON_SIZE_MENU)
		item = gtk.ImageMenuItem(txt)
		item.set_image(img)
	        item.connect("activate", cbmethod)
		return item

	def __init__(self, repo_journal, repo_accounts, sync_handler, syncweb_handler, init_handler):
		self.labels = []
		lines = []
		
		lines.append(gtk.Label())

		self.datatable = gtk.Table(5,2)
		self.datatable.set_col_spacings(8)
		self.datatable.set_row_spacings(8)
		lines.append(self.datatable)

		lines.append(gtk.Label())

		self.navbuttonsbox = gtk.HBox(True)
		lines.append(self.navbuttonsbox)

		self.repo_journal = repo_journal
		self.repo_accounts = repo_accounts
		self.sync_handler = sync_handler
		self.syncweb_handler = syncweb_handler
		self.init_handler = init_handler

		self._accountlist = gtk.ListStore(str, int)

		field_db = gtk.ComboBox(self._accountlist);
		field_cr = gtk.ComboBox(self._accountlist);

		cell = gtk.CellRendererText()
		field_db.clear()
		field_db.pack_start(cell)
		field_db.set_attributes(cell, text=0)
		field_db.set_active(0)

		cell = gtk.CellRendererText()
		field_cr.clear()
		field_cr.pack_start(cell)
		field_cr.set_attributes(cell, text=0)
		field_cr.set_active(0)

		field_date = gtk.Entry()
		field_value = gtk.Entry()
		field_description = gtk.Entry()

		# field_value.set_property('xalign', 1)

		self._datafield(u"Debit", field_db, 20, 0)
		self._datafield(u"Credit", field_cr, 20, 0)
		self._datafield(u"Date", field_date, 10, 0)
		self._datafield(u"$", field_value, 12, 1)
		self._datafield(u"Description", field_description, 30, 0, 60)

		self.fields = {}
		self.fields["db"] = field_db
		self.fields["cr"] = field_cr
		self.fields["date"] = field_date
		self.fields["value"] = field_value
		self.fields["description"] = field_description

		lines.append(gtk.Label())

		self.field_msg = gtk.Label()
		self.field_msg.show()
		lines.append(self.field_msg)

		self.buttons = {}
		self.buttons["first"] = self._navbutton(gtk.STOCK_GOTO_FIRST, self.first)
		self.buttons["left"] = self._navbutton(gtk.STOCK_GO_BACK, self.left)
		self.buttons["right"] = self._navbutton(gtk.STOCK_GO_FORWARD, self.right)
		self.buttons["last"] = self._navbutton(gtk.STOCK_GOTO_LAST, self.last)
		self.buttons["add"] = self._navbutton(gtk.STOCK_ADD, self.add)
		self.buttons["remove"] = self._navbutton(gtk.STOCK_REMOVE, self.remove)
		self.buttons["alt"] = self._navbutton(gtk.STOCK_EDIT, self.alt)
		self.buttons["yes"] = self._navbutton(gtk.STOCK_OK, self.yes)
		self.buttons["no"] = self._navbutton(gtk.STOCK_CANCEL, self.no)

		if is_hildon:
			self.app = hildon.Program()
			self.window = hildon.Window()
		else:
			self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
			self.window.set_border_width(5)
			
		self.window.set_title("Luca Mobile")
		gtk.set_application_name("")
		self.window.show_all()

		self.menu = gtk.Menu()
		self.window.set_menu(self.menu)
		self.menu.show_all()

		self.menu_SYNC = self._menuitem(gtk.STOCK_NETWORK, "Sync via Bluetooth", self.menu_sync)
		self.menu_SYNCWEB = self._menuitem(gtk.STOCK_NETWORK, "Sync via Web", self.menu_syncweb)
		self.menu_QUIT = self._menuitem(gtk.STOCK_QUIT, "_Quit", self.menu_quit)
		self.menu_ADVANCED = gtk.Menu()
		self.menu_ADVANCED_item = gtk.MenuItem("Advanced")
		self.menu_ADVANCED_item.set_submenu(self.menu_ADVANCED)
		self.menu_INIT = self._menuitem(gtk.STOCK_NEW, "Reset database", self.menu_init)

		self.menu.add(self.menu_SYNC)
		self.menu.add(self.menu_SYNCWEB)
		self.menu.add(self.menu_ADVANCED_item)
		self.menu_ADVANCED.add(self.menu_INIT)
		self.menu.add(self.menu_QUIT)

		self.mainpack = gtk.VBox()
		for l in lines:
			self.mainpack.pack_start(l, False, False, 0)
			l.show()

		self.mainpack.show()

		self.window.connect("delete_event", self.delete_event)
		self.window.connect("destroy", self.destroy)
		self.window.set_border_width(5)
		self.window.add(self.mainpack)

		self.mode = BROWSE
		self.journal_position = 0

		self.window.show_all()
		self.menu.show_all()

		self.msg(u"")
		self.load_accounts()
		self.refresh_journal()
	
	def main(self):
		gtk.main()

